@@ -1,6 +1,6 @@
 // apt_options.cc
 //
-//   Copyright (C) 2000, 2007 Daniel Burrows
+//   Copyright (C) 2000, 2007-2008 Daniel Burrows
 //
 //   This program is free software; you can redistribute it and/or
 //   modify it under the terms of the GNU General Public License as
@@ -63,8 +63,8 @@ namespace aptitude
 struct option_item
 {
   enum {OPTION_BOOL, OPTION_STRING, OPTION_RADIO, OPTION_END} type;
-  const char *description;
-  const char *long_description;
+  const char *untranslated_description;
+  const char *untranslated_long_description;
   const char *option_name;
 
   union
@@ -79,28 +79,28 @@ struct option_item
   vector<radio_choice> choices;
 
   option_item()
-    :type(OPTION_END), description(NULL), option_name(NULL)
+    :type(OPTION_END), untranslated_description(NULL), option_name(NULL)
   {
   }
 
-  option_item(const char *_description,
-	      const char *_long_description,
+  option_item(const char *_untranslated_description,
+	      const char *_untranslated_long_description,
 	      const char *_option_name, bool def)
     :type(OPTION_BOOL),
-     description(_description),
-     long_description(_long_description),
+     untranslated_description(_untranslated_description),
+     untranslated_long_description(_untranslated_long_description),
      option_name(_option_name),
      b_default(def)
   {
   }
 
-  option_item(const char *_description,
-	      const char *_long_description,
+  option_item(const char *_untranslated_description,
+	      const char *_untranslated_long_description,
 	      const char *_option_name,
 	      const char *def)
     :type(OPTION_STRING),
-     description(_description),
-     long_description(_long_description),
+     untranslated_description(_untranslated_description),
+     untranslated_long_description(_untranslated_long_description),
      option_name(_option_name),
      s_default(def)
   {
@@ -109,8 +109,8 @@ struct option_item
   /** \brief Construct a radio item from a list of choices,
    *  destroying each choice after it's inserted into the list.
    */
-  static option_item radio(const char *description,
-			   const char *long_description,
+  static option_item radio(const char *untranslated_description,
+			   const char *untranslated_long_description,
 			   const char *option_name,
 			   const char *def,
 			   const radio_choice *choice_1,
@@ -122,9 +122,9 @@ struct option_item
     va_start(args, choice_1);
 
     rval.type=OPTION_RADIO;
-    rval.description=description;
-    rval.long_description = long_description;
-    rval.option_name=option_name;
+    rval.untranslated_description = untranslated_description;
+    rval.untranslated_long_description = untranslated_long_description;
+    rval.option_name = option_name;
     rval.s_default=def;
 
     rval.choices.push_back(*choice_1);
@@ -148,12 +148,12 @@ struct option_item
 };
 
 	radio_choice *choice(const char *value,
-			     const char *description,
-			     const char *long_description)
+			     const char *untranslated_description,
+			     const char *untranslated_long_description)
 	{
 	  return new radio_choice(value,
-				  W_(description),
-				  W_(long_description));
+				  untranslated_description,
+				  untranslated_long_description);
 	}
 
 option_item ui_options[]={
@@ -196,24 +196,34 @@ option_item ui_options[]={
 		     N_("This option controls whether aptitude will wait for confirmation after a download before it goes ahead and installs packages."),
 		     PACKAGE "::UI::Pause-After-Download",
 		     "OnlyIfError",
-		     choice("No", _("Never"),
-			    _("Never wait for the user after downloading packages: always begin the installation immediately.")),
-		     choice("OnlyIfError", _("When an error occurs"),
-			    _("Wait for confirmation if an error occurred during the download.  If there were no errors, begin installing packages immediately.")),
-		     choice("Yes", _("Always"),
-			    _("Always wait for the user to confirm the download before proceeding with the installation.")),
+		     choice("No", N_("Never"),
+			    N_("Never wait for the user after downloading packages: always begin the installation immediately.")),
+		     choice("OnlyIfError", N_("When an error occurs"),
+			    N_("Wait for confirmation if an error occurred during the download.  If there were no errors, begin installing packages immediately.")),
+		     choice("Yes", N_("Always"),
+			    N_("Always wait for the user to confirm the download before proceeding with the installation.")),
 		     NULL),
   option_item(N_("Use a 'status-line' download indicator for all downloads"),
 	      N_("If this option is enabled, aptitude will display "
 		 "the status of ongoing downloads at the bottom of "
 		 "the screen, rather than opening a new view."),
 	      PACKAGE "::UI::Minibuf-Download-Bar", false),
-  option_item(N_("Display the extended description area by default"),
-	      N_("If this option is enabled, the long description "
+  option_item(N_("Display the information area by default"),
+	      N_("If this option is enabled, the information "
 		 "area (the pane at the bottom of the screen) in the "
 		 "package list will be visible when the program "
 		 "starts; otherwise, it will be initially hidden."),
 	      PACKAGE "::UI::Description-Visible-By-Default", true),
+  option_item(N_("Display tabs for the available views"),
+	      N_("If this option is enabled, tabs will appear at the top "
+		 "of the screen listing the currently opened views."),
+	      PACKAGE "::UI::ViewTabs", true),
+  option_item(N_("Display tabs for the information area"),
+	      N_("If this option is enabled, tabs will appear at the top of "
+		 "the information area (the pane at the bottom of the screen) "
+		 "listing the different displays of information that can be "
+		 "viewed there."),
+	      PACKAGE "::UI::InfoAreaTabs", false),
   option_item(N_("Advance to the next item after changing the state of a package"),
 	      N_("If this option is enabled, then performing an "
 		 "action on a package (for instance, installing or "
@@ -278,8 +288,8 @@ option_item misc_options[]={
   option_item(N_("URL to use to download changelogs"),
 	      N_("This option controls the template that's used to "
 		 "download changelogs from the Debian Web site.  You "
-		 "should only need to change this if the URL of "
-		 "the package archive changes."),
+		 "should only need to change this if the changelogs move "
+		 "to a different URL."),
 	      PACKAGE "::Changelog-URL-Template",
 	      "http://cgi.debian.org/cgi-bin/get-changelog?package=%s"),
   option_item(N_("Display a preview of what will be done before doing it"),
@@ -345,12 +355,12 @@ option_item dependency_options[]={
 	      N_("If this option is enabled, and you perform an "
 		 "install run while some packages are broken, "
 		 "aptitude will automatically apply the current "
-		 "suggestion of the problem resolver.  Otherwise,"
+		 "suggestion of the problem resolver.  Otherwise, "
 		 "aptitude will prompt you for a solution to the "
 		 "broken dependencies."),
 	      PACKAGE "::Auto-Fix-Broken", true),
   option_item(N_("Install recommended packages automatically"),
-	      N_("If this option is enabled  and \"automatically "
+	      N_("If this option is enabled and \"automatically "
 		 "resolve dependencies\" is also enabled, aptitude "
 		 "will attempt to install the recommendations of "
 		 "newly installed packages in addition to their "
@@ -360,8 +370,8 @@ option_item dependency_options[]={
 		 "If this option is enabled and \"Remove unused "
 		 "packages automatically\" is enabled, packages "
 		 "that are recommended by an installed package "
-		 "will not automatically be removed."),
-	      PACKAGE "::Recommends-Important", true),
+		 "will not be automatically removed."),
+	      "Apt::Install-Recommends", true),
   option_item(N_("Remove unused packages automatically"),
 	      N_("If this option is enabled, packages that are "
 		 "automatically installed and that no manually "
@@ -380,6 +390,21 @@ option_item dependency_options[]={
 		 "depends on them: they will never be targeted "
 		 "for removal as unused packages."),
 	      PACKAGE "::Keep-Unused-Pattern", ""),
+  option_item(N_("Allow dependency resolutions that break holds or forbids"),
+	      N_("If this option is enabled, then whenever aptitude "
+		 "solves a dependency problem, it will consider "
+		 "modifying held packages, or installing forbidden "
+		 "package versions.  With this option disabled, these "
+		 "solutions will be rejected by default."
+		 "\n"
+		 "NOTE: At present, this restriction only applies to situations "
+		 "in which aptitude's dependency resolver (the red bar "
+		 "at the bottom of the screen) is activated.  aptitude "
+		 "will still break holds when automatically installing "
+		 "the dependencies of a package that has just been "
+		 "selected for installation or upgrade, due to apt bug "
+		 "#470035."),
+	      PACKAGE "::ProblemResolver::Allow-Break-Holds", false),
   option_item()
 };
 
@@ -410,24 +435,29 @@ option_item dependency_options[]={
 	  }
 	};
 
+	/** \brief Instantiate a widget for the given option description.
+	 *
+	 *  Any untranslated strings in the structure are translated
+	 *  by calls to gettext().
+	 */
 	cw::treeitem *parse_option(const option_item &option)
 	{
 	  eassert(option.type != option_item::OPTION_END);
 	  switch(option.type)
 	    {
 	    case option_item::OPTION_BOOL:
-	      return make_boolean_item(W_(option.description),
-				       W_(option.long_description),
+	      return make_boolean_item(W_(option.untranslated_description),
+				       _(option.untranslated_long_description),
 				       option.option_name,
 				       option.b_default);
 	    case option_item::OPTION_STRING:
-	      return make_string_item(W_(option.description),
-				      W_(option.long_description),
+	      return make_string_item(W_(option.untranslated_description),
+				      _(option.untranslated_long_description),
 				      option.option_name,
 				      option.s_default);
 	    case option_item::OPTION_RADIO:
-	      return make_radio_item(W_(option.description),
-				     W_(option.long_description),
+	      return make_radio_item(W_(option.untranslated_description),
+				     _(option.untranslated_long_description),
 				     option.option_name,
 				     option.choices,
 				     option.s_default);
@@ -505,7 +535,12 @@ option_item dependency_options[]={
 
 	    // Use an empty label to produce a "bar" dividing the two
 	    // halves of the screen.
-	    cw::label_ref middle_label = cw::label::create("", cw::get_style("Status"));
+
+	    // These declarations are workarounds for strange behavior
+	    // in g++ 4.3.
+	    std::string empty_label;
+	    cw::style status_style(cw::get_style("Status"));
+	    cw::label_ref middle_label = cw::label::create(empty_label, status_style);
 	    desc_area = cw::text_layout::create();
 
 	    cw::scrollbar_ref desc_area_scrollbar = cw::scrollbar::create(cw::scrollbar::VERTICAL);
