@@ -58,7 +58,6 @@ namespace
       }
 
     bool rval = true;
-    bool last_was_or = false;
     typedef pkgSrcRecords::Parser::BuildDepRec BuildDepRec;
     typedef std::vector<BuildDepRec> BuildDepList;
     for(BuildDepList::const_iterator it = sourcepkg.get_build_deps().begin();
@@ -108,7 +107,7 @@ namespace
 		    pkgCache::PkgIterator pkg = (*apt_cache_file)->FindPkg(it2->Package);
 		    if(!pkg.end())
 		      {
-			int best_priority;
+			int best_priority = -1;
 			pkgCache::VerIterator best_ver;
 			for(pkgCache::VerIterator ver = pkg.VersionList();
 			    !ver.end(); ++ver)
@@ -293,7 +292,7 @@ bool cmdline_applyaction(cmdline_pkgaction_type action,
     }
 
   pkgCache::VerIterator ver=pkg.CurrentVer();
-  if(action==cmdline_install)
+  if(action==cmdline_install || action == cmdline_installauto)
     ver=cmdline_find_ver(pkg, source, sourcestr);
 
   switch(action)
