@@ -2,7 +2,6 @@
 //
 //   Copyright 2004 Daniel Burrows
 //
-// Stuff that doesn't particularly belong anywhere.
 
 #ifndef CMDLINE_COMMON_H
 #define CMDLINE_COMMON_H
@@ -13,6 +12,11 @@
 #include <string>
 #include <vector>
 
+/** \brief Stuff that doesn't particularly belong anywhere.
+ * 
+ *  \file cmdline_common.h
+ */
+
 extern int screen_width;
 void update_screen_width(int foo=0);
 
@@ -25,7 +29,7 @@ enum cmdline_pkgaction_type
   {cmdline_install, cmdline_installauto, cmdline_remove,
    cmdline_purge, cmdline_hold, cmdline_unhold, cmdline_markauto,
    cmdline_unmarkauto, cmdline_forbid_version, cmdline_reinstall,
-   cmdline_keep};
+   cmdline_keep, cmdline_build_depends};
 
 enum cmdline_version_source {cmdline_version_cand,
 			     /** \brief The current version, if any;
