@@ -45,6 +45,11 @@ string StdinEOFException::errmsg() const
 struct fetchinfo
 {
   double FetchBytes, FetchPBytes, DebBytes;
+
+  fetchinfo()
+    : FetchBytes(0), FetchPBytes(0), DebBytes(0)
+  {
+  }
 };
 
 static bool pkg_byname_compare(const pkgCache::PkgIterator &a, const pkgCache::PkgIterator &b)
@@ -640,12 +645,13 @@ static bool prompt_essential()
     {
       printf(_("WARNING: Performing this action will probably cause your system to break!\n         Do NOT continue unless you know EXACTLY what you are doing!\n"));
 
-      string prompt=_("I am aware that this is a very bad idea");
+      string untranslated_prompt = N_("I am aware that this is a very bad idea");
+      string prompt = _(untranslated_prompt.c_str());
       char buf[1024];
 
       printf(_("To continue, type the phrase \"%s\":\n"), prompt.c_str());
       cin.getline(buf, 1023);
-      bool rval=(prompt==buf);
+      bool rval = (prompt == buf || untranslated_prompt == buf);
 
       while(!cin && !cin.eof())
 	cin.getline(buf, 1023);
@@ -714,6 +720,13 @@ static bool prompt_trust()
 
       const string okstr=_("Yes"), abortstr=_("No");
 
+      // These strings are used to compare in a translation-invariant
+      // way, so that "yes" and "no" are always valid inputs; if the
+      // user can't enter the translated string for some reason,
+      // he/she can always enter the fallback strings.
+      const string fallback_okstr = "Yes";
+      const string fallback_abortstr = "No";
+
       while(1)
 	{
 	  printf(_("Do you want to ignore this warning and proceed anyway?\n"));
@@ -725,12 +738,15 @@ static bool prompt_trust()
 	  if(cin.eof())
 	    throw StdinEOFException();
 
-	  const bool is_ok=(strncasecmp(okstr.c_str(), buf, okstr.size())==0);
-	  const bool is_abort=(strncasecmp(abortstr.c_str(), buf, abortstr.size())==0);
 
-	  const bool rval=is_ok;
+	  const bool is_ok =             strncasecmp(okstr.c_str(), buf, okstr.size()) == 0;
+	  const bool is_fallback_ok =    strncasecmp(fallback_okstr.c_str(), buf, fallback_okstr.size()) == 0;
+	  const bool is_abort =          strncasecmp(abortstr.c_str(), buf, abortstr.size()) == 0;
+	  const bool is_fallback_abort = strncasecmp(fallback_abortstr.c_str(), buf, fallback_abortstr.size()) == 0;
 
-	  if(!is_ok && !is_abort)
+	  const bool rval = is_ok || (is_fallback_ok && !is_abort);
+
+	  if(!is_ok && !is_abort && !is_fallback_ok && !is_fallback_abort)
 	    printf(_("Unrecognized input.  Enter either \"%s\" or \"%s\".\n"), okstr.c_str(), abortstr.c_str());
 	  else
 	    return rval;
@@ -748,11 +764,11 @@ static bool prompt_trust()
  *  user are being installed/removed (eg, because of sticky states) and
  *  tell the caller to pause for confirmation.
  */
-static bool cmdline_show_preview(bool as_upgrade, pkgset &to_install,
-				 pkgset &to_hold, pkgset &to_remove,
-				 bool showvers, bool showdeps,
-				 bool showsize, bool showwhy,
-				 int verbose)
+bool cmdline_show_preview(bool as_upgrade, pkgset &to_install,
+			  pkgset &to_hold, pkgset &to_remove,
+			  bool showvers, bool showdeps,
+			  bool showsize, bool showwhy,
+			  int verbose)
 {
   const int quiet = aptcfg->FindI("Quiet", 0);
   bool all_empty=true;
