@@ -197,8 +197,7 @@ static void resolver_help(ostream &out)
 			      flowindentbox(0, 15,
 					    cw::fragf(_("reject the given package versions; don't display any solutions in which they occur.  Enter UNINST instead of a version to reject removing the package."))),
 			      flowindentbox(0, 15,
-							// ForTranslators: "(;" is ...???? A smiley?
-					    cw::fragf(_("accept the given package versions (; display only solutions in which they occur.  Enter UNINST instead of a version to accept removing the package."))),
+					    cw::fragf(_("accept the given package versions; display only solutions in which they occur.  Enter UNINST instead of a version to accept removing the package."))),
 			      flowindentbox(0, 3,
 					    cw::fragf(_("adjust the state of the listed packages, where ACTION is one of:"))),
 			      flowindentbox(0, 4,
@@ -236,7 +235,7 @@ static pkgCache::VerIterator choose_version(const vector<pkgCache::VerIterator>
 
   for(vector<pkgCache::VerIterator>::size_type i = 0;
       i < choices.size(); ++i)
-    cout << ssprintf(" (%d) %s", i+1, archives_text(choices[i]).c_str()) << endl;
+    cout << ssprintf(" (%d) %s", (int)(i+1), archives_text(choices[i]).c_str()) << endl;
 
   while(1)
     {
@@ -246,8 +245,8 @@ static pkgCache::VerIterator choose_version(const vector<pkgCache::VerIterator>
       istringstream in(response);
       in >> ws >> i >> ws;
 
-      if(!in || !in.eof() || i < 1 || i > choices.size())
-	cerr << ssprintf(_("Invalid response.  Please enter an integer between 1 and %d."), choices.size()) << endl;
+      if(!in || !in.eof() || i < 1 || i > (signed)choices.size())
+	cerr << ssprintf(_("Invalid response.  Please enter an integer between 1 and %d."), (int)choices.size()) << endl;
       else
 	return choices[i];
     }
@@ -560,7 +559,7 @@ aptitude_solution calculate_current_solution(bool suppress_message)
 	{
 	  resolver_manager::state state = resman->state_snapshot();
 
-	  spin.set_msg(ssprintf(_("open: %d; closed: %d; defer: %d; conflict: %d"),
+	  spin.set_msg(ssprintf(_("open: %zd; closed: %zd; defer: %zd; conflict: %zd"),
 				state.open_size, state.closed_size,
 				state.deferred_size, state.conflicts_size));
 	  spin.display();
@@ -649,7 +648,7 @@ cmdline_resolve_deps(pkgset &to_install,
 		    break;
 		  case 'N':
 		    {
-		      int curr_count = resman->generated_solution_count();
+		      unsigned int curr_count = resman->generated_solution_count();
 
 		      if(curr_count>0)
 			while(resman->get_selected_solution() < curr_count)
