@@ -5,6 +5,7 @@
 #include "cmdline_search.h"
 
 #include "cmdline_common.h"
+#include "cmdline_util.h"
 
 #include <aptitude.h>
 #include <load_sortpolicy.h>
@@ -27,8 +28,9 @@ using namespace std;
 namespace cw = cwidget;
 using cwidget::util::transcode;
 using namespace aptitude::matching;
+using namespace cwidget::config;
 
-class search_result_parameters : public cwidget::config::column_parameters
+class search_result_parameters : public column_parameters
 {
   pkg_match_result *r;
 public:
@@ -91,7 +93,8 @@ public:
 
 // FIXME: apt-cache does lots of tricks to make this fast.  Should I?
 int cmdline_search(int argc, char *argv[], const char *status_fname,
-		   string display_format, string width, string sort)
+		   string display_format, string width, string sort,
+		   bool disable_columns)
 {
   int real_width=-1;
 
@@ -123,7 +126,7 @@ int cmdline_search(int argc, char *argv[], const char *status_fname,
       return -1;
     }
 
-  cwidget::config::column_definition_list *columns =
+  column_definition_list *columns =
     parse_columns(wdisplay_format,
 		  pkg_item::pkg_columnizer::parse_column_type,
 		  pkg_item::pkg_columnizer::defaults);
@@ -201,15 +204,18 @@ int cmdline_search(int argc, char *argv[], const char *status_fname,
   for(vector<pair<pkgCache::PkgIterator, pkg_match_result *> >::iterator i=output.begin();
       i!=output.end(); ++i)
     {
-      cwidget::config::column_parameters *p =
+      column_parameters *p =
 	new search_result_parameters(i->second);
-
-      printf("%ls\n",
-	     pkg_item::pkg_columnizer(i->first,
-				      i->first.VersionList(),
-				      *columns,
-				      0).layout_columns(real_width==-1?screen_width:real_width,
-							*p).c_str());
+      pkg_item::pkg_columnizer columnizer(i->first,
+					  i->first.VersionList(),
+					  *columns,
+					  0);
+      if(disable_columns)
+	printf("%ls\n", aptitude::cmdline::de_columnize(*columns, columnizer, *p).c_str());
+      else
+	printf("%ls\n",
+	       columnizer.layout_columns(real_width==-1?screen_width:real_width,
+					 *p).c_str());
 
       // Note that this deletes the whole result, so we can't re-use
       // the list.
