@@ -384,7 +384,7 @@ static cw::fragment *version_file_fragment(pkgCache::VerIterator ver,
   if(!pkg.CurrentVer().end())
     fragments.push_back(cw::fragf("%s: %s%n", _("Automatically installed"),
 			      (state.Flags & pkgCache::Flag::Auto)
-			      ? _("no") : _("yes")));
+			      ? _("yes") : _("no")));
 
   fragments.push_back(cw::fragf("%s%s%n", _("Version: "), ver.VerStr()));
   fragments.push_back(cw::fragf("%s%s%n", _("Priority: "),
@@ -530,7 +530,7 @@ bool do_cmdline_show(string s, int verbose)
       sourcestr = default_release;
     }
 
-  bool is_pattern=(name.find('~')!=name.npos);
+  bool is_pattern = cmdline_is_search_pattern(name);
   pkgCache::PkgIterator pkg;
 
   if(!is_pattern)
@@ -548,6 +548,8 @@ bool do_cmdline_show(string s, int verbose)
     return do_cmdline_show_target(pkg, source, sourcestr, verbose, has_explicit_source);
   else if(is_pattern)
     {
+      using namespace aptitude::matching;
+
       std::auto_ptr<pkg_matcher> m(parse_pattern(name));
 
       if(m.get() == NULL)
@@ -558,7 +560,7 @@ bool do_cmdline_show(string s, int verbose)
 
       for(pkgCache::PkgIterator P=(*apt_cache_file)->PkgBegin();
 	  !P.end(); ++P)
-	if(m->matches(P, *apt_cache_file, *apt_package_records))
+	if(apply_matcher(m.get(), P, *apt_cache_file, *apt_package_records))
 	  {
 	    if(!do_cmdline_show_target(P, source, sourcestr, verbose, has_explicit_source))
 	      return false;
