@@ -1,6 +1,6 @@
 // cmdline_update.cc
 //
-//   Copyright (C) 2004-2006 Daniel Burrows
+//   Copyright (C) 2004-2008 Daniel Burrows
 //
 //   This program is free software; you can redistribute it and/or
 //   modify it under the terms of the GNU General Public License as
@@ -26,14 +26,13 @@
 #include <generic/apt/download_update_manager.h>
 
 #include <apt-pkg/error.h>
-#include <apt-pkg/progress.h>
 
 void print_autoclean_msg()
 {
   printf(_("Deleting obsolete downloaded files\n"));
 }
 
-int cmdline_update(int argc, char *argv[])
+int cmdline_update(int argc, char *argv[], int verbose)
 {
   _error->DumpErrors();
 
@@ -43,18 +42,15 @@ int cmdline_update(int argc, char *argv[])
       return -1;
     }
 
-  OpTextProgress progress(aptcfg->FindI("Quiet", 0));
-
-  if(_error->PendingError())
-    {
-      _error->DumpErrors();
-      return -1;
-    }
+  // Don't exit if there's an error: it probably means that there
+  // was a problem loading the package lists, so go ahead and try to
+  // download new ones.
+  _error->DumpErrors();
 
   download_update_manager m;
   m.pre_autoclean_hook.connect(sigc::ptr_fun(print_autoclean_msg));
   int rval =
-    (cmdline_do_download(&m) == download_manager::success ? 0 : -1);
+    (cmdline_do_download(&m, verbose) == download_manager::success ? 0 : -1);
 
   if(_error->PendingError())
     rval = -1;
