@@ -361,6 +361,8 @@ namespace aptitude
 	return cw::fragf(_("Remove(%s provides %s)"),
 			 const_cast<pkgCache::PrvIterator &>(prv).OwnerPkg().Name(),
 			 mpkg.Name());
+      default:
+	return cw::text_fragment("Internal error: bad target node type.");
       }
   }
 
@@ -639,6 +641,7 @@ namespace aptitude
 	    seen_packages[i] = other.seen_packages[i];
 	}
       first_iteration = other.first_iteration;
+      return *this;
     }
 
     ~justification_search()
