@@ -1,6 +1,6 @@
 // desc_parse.cc
 //
-//  Copyright 2004-2006 Daniel Burrows
+//  Copyright 2004-2008 Daniel Burrows
 //
 //   This program is free software; you can redistribute it and/or
 //   modify it under the terms of the GNU General Public License as
@@ -284,21 +284,60 @@ cw::fragment *make_tags_fragment(const pkgCache::PkgIterator &pkg)
   if(pkg.end())
     return NULL;
 
-  const set<tag> *s = get_tags(pkg);
+#ifdef HAVE_EPT
+  typedef ept::debtags::Tag tag;
+  using aptitude::apt::get_tags;
+#endif
+
+#ifdef HAVE_EPT
+  const set<tag> realS(get_tags(pkg));
+  const set<tag> * const s(&realS);
+#else
+  const set<tag> * const s(get_tags(pkg));
+#endif
+
+  vector<cw::fragment *> rval;
   if(s != NULL && !s->empty())
     {
       vector<cw::fragment *> tags;
-
       for(set<tag>::const_iterator i = s->begin(); i != s->end(); ++i)
-	tags.push_back(cw::text_fragment(i->str(), cw::style_attrs_on(A_BOLD)));
+	{
+#ifdef HAVE_EPT
+	  std::string name(i->fullname());
+#else
+	  const std::string name(i->str());
+#endif
+
+	  tags.push_back(cw::text_fragment(name, cw::style_attrs_on(A_BOLD)));
+	}
 
       wstring tagstitle = W_("Tags");
 
-      return cw::fragf("%ls: %F",
-		   tagstitle.c_str(),
-		   indentbox(0, wcswidth(tagstitle.c_str(), tagstitle.size())+2,
+      rval.push_back(cw::fragf("%ls: %F",
+			       tagstitle.c_str(),
+			       indentbox(0, wcswidth(tagstitle.c_str(), tagstitle.size())+2,
+					 wrapbox(cw::join_fragments(tags, L", ")))));
+    }
+
+  typedef aptitudeDepCache::user_tag user_tag;
+  const set<user_tag> &user_tags((*apt_cache_file)->get_ext_state(pkg).user_tags);
+  if(!user_tags.empty())
+    {
+      vector<cw::fragment *> tags;
+      for(set<user_tag>::const_iterator it = user_tags.begin();
+	  it != user_tags.end(); ++it)
+	{
+	  tags.push_back(cw::text_fragment((*apt_cache_file)->deref_user_tag(*it),
+					   cw::style_attrs_on(A_BOLD)));
+	}
+
+      wstring title = W_("User Tags");
+      rval.push_back(dropbox(cw::fragf("%ls: ", title.c_str()),
 			     wrapbox(cw::join_fragments(tags, L", "))));
     }
+
+  if(!rval.empty())
+    return cw::join_fragments(rval, L"\n");
   else
     return NULL;
 }
