@@ -1,6 +1,6 @@
 // apt.cc
 //
-//  Copyright 1999-2007 Daniel Burrows
+//  Copyright 1999-2008 Daniel Burrows
 //
 //  This program is free software; you can redistribute it and/or modify
 //  it under the terms of the GNU General Public License as published by
@@ -73,6 +73,10 @@ resolver_manager *resman = NULL;
 string *pendingerr=NULL;
 bool erroriswarning=false;
 
+// Set to "true" if we have a version of the apt library with
+// support for overriding configuration settings via RootDir.
+static bool apt_knows_about_rootdir;
+
 static Configuration *theme_config;
 static Configuration *user_config;
 
@@ -110,16 +114,67 @@ static void reload_user_pkg_hier()
     }
 }
 
+bool get_apt_knows_about_rootdir()
+{
+  return apt_knows_about_rootdir;
+}
+
 void apt_preinit()
 {
+  // The old name for the recommends-should-be-automatically-installed
+  // setting and the new one.
+  const char * const aptitudeIgnoreRecommendsImportant = PACKAGE "::Ignore-Recommends-Important";
+  const char * const aptitudeRecommendsImportant = PACKAGE "::Recommends-Important";
+  const char * const aptInstallRecommends = "APT::Install-Recommends";
+
   signal(SIGPIPE, SIG_IGN);
 
+  // Probe apt to see if it has RootDir support.
+  {
+    Configuration tmp;
+    config_change_pusher push1("RootDir", "/a/b/c/d", tmp);
+    config_change_pusher push2("Dir::ASDF", "/x/y/z", tmp);
+
+    std::string found_loc = tmp.FindFile("Dir::ASDF");
+
+    std::string real_found_loc;
+    for(std::string::const_iterator it = found_loc.begin();
+	it != found_loc.end(); ++it)
+      {
+	// Drop repeated slashes.
+	if(*it != '/' || real_found_loc.size() == 0 ||
+	   real_found_loc[real_found_loc.size() - 1] != '/')
+	  real_found_loc.push_back(*it);
+      }
+
+    apt_knows_about_rootdir =
+      (real_found_loc == "/a/b/c/d/x/y/z");
+  }
+
   theme_config=new Configuration;
   user_config=new Configuration;
 
   ReadConfigFile(*theme_config, PKGDATADIR "/aptitude-defaults");
 
   pkgInitConfig(*_config);
+
+
+  ReadConfigFile(*_config, PKGDATADIR "/section-descriptions");
+
+  // ForTranslators: Set this string to the name of a configuration
+  // file in $pkgdatadir/aptitude that overrides defaults for your
+  // language.  This is particularly intended for overriding entries
+  // in the Aptitude::Sections::Descriptions tree.
+  //
+  // For instance, Sections localized for the language .ww might be
+  // stored in a file named aptitude-defaults.ww, which would be
+  // indicated by translating "Localized defaults|" below to
+  // "aptitude-defaults.ww".  If you use this mechanism, you should
+  // also add your defaults file to pkgdata_DATA in Makefile.am.
+  std::string localized_config_name = P_("Localized defaults|");
+  if(localized_config_name.size() > 0)
+    ReadConfigFile(*_config, PKGDATADIR "/" + localized_config_name);
+
   pkgInitSystem(*_config, _system);
 
   // Allow a user-specific customization file.
@@ -145,7 +200,25 @@ void apt_preinit()
 
   aptcfg=new signalling_config(user_config, _config, theme_config);
 
-  aptcfg->connect(PACKAGE "::Recommends-Important",
+  // If the user has a Recommends-Important setting and has allowed us
+  // to read it by seting Ignore-Recommends-Important to false,
+  // migrate it over and then set Ignore-Recommends-Important to true.
+  if(!aptcfg->FindB(aptitudeIgnoreRecommendsImportant, false) &&
+     aptcfg->Exists(aptitudeRecommendsImportant))
+    {
+      // If it was overridden to "false" and the system setting for
+      // APT::Install-Recommends is "true", set the latter to "false"
+      // to preserve aptitude's behavior.
+      if(!aptcfg->FindB(aptitudeRecommendsImportant, true) &&
+	 aptcfg->FindB(aptInstallRecommends, true))
+	aptcfg->Set(aptInstallRecommends, "false");
+
+      aptcfg->Set(aptitudeIgnoreRecommendsImportant, "true");
+
+      apt_dumpcfg(PACKAGE);
+    }
+
+  aptcfg->connect("APT::Install-Recommends",
 		  sigc::ptr_fun(&reset_interesting_dep_memoization));
 
   cache_closed.connect(sigc::ptr_fun(&reset_interesting_dep_memoization));
@@ -308,7 +381,7 @@ void apt_load_cache(OpProgress *progress_bar, bool do_initselections,
 				  false, status_fname);
 
       if(!open_failed)
-	_error->Warning(_("Could not lock the cache file.  Opening in read-only mode; any changes you make to the states of packages will NOT be preserved!"));
+	_error->Warning(_("Could not lock the cache file; this usually means that dpkg or another apt tool is already installing packages.  Opening in read-only mode; any changes you make to the states of packages will NOT be preserved!"));
     }
 
   if(open_failed)
@@ -333,7 +406,11 @@ void apt_load_cache(OpProgress *progress_bar, bool do_initselections,
   apt_undos->clear_items();
 
   load_tasks(*progress_bar);
+#ifndef HAVE_EPT
   load_tags(*progress_bar);
+#else
+  aptitude::apt::load_tags();
+#endif
 
   if(user_pkg_hier)
     {
@@ -781,7 +858,7 @@ static bool internal_is_interesting_dep(const pkgCache::DepIterator &d,
   else if(const_cast<pkgCache::DepIterator &>(d).IsCritical())
     return true;
   else if(d->Type != pkgCache::Dep::Recommends ||
-	  !aptcfg->FindB(PACKAGE "::Recommends-Important", true))
+	  !aptcfg->FindB("Apt::Install-Recommends", true))
     return false;
   else
     {
@@ -930,3 +1007,55 @@ std::wstring get_long_description(const pkgCache::VerIterator &ver,
     return cwidget::util::transcode(records->Lookup(df).LongDesc());
 #endif
 }
+
+namespace aptitude
+{
+  namespace apt
+  {
+    bool is_full_replacement(const pkgCache::DepIterator &dep)
+    {
+      if(dep.end())
+	return false;
+
+      if(dep->Type != pkgCache::Dep::Replaces)
+	return false;
+
+      if((dep->CompareOp & ~pkgCache::Dep::Or) != pkgCache::Dep::NoOp)
+	return false;
+
+      pkgCache::PkgIterator target = const_cast<pkgCache::DepIterator &>(dep).TargetPkg();
+
+      // Check whether the parent of the dep provides this target and
+      // conflicts with it.
+
+      bool found_provides = false;
+
+      for(pkgCache::PrvIterator prv = const_cast<pkgCache::DepIterator &>(dep).ParentVer().ProvidesList();
+	  !found_provides && !prv.end(); ++prv)
+	{
+	  if(prv.ParentPkg() == target)
+	    found_provides = true;
+	}
+
+      if(!found_provides)
+	return false;
+
+      bool found_conflicts = false;
+
+      for(pkgCache::DepIterator possible_conflict = const_cast<pkgCache::DepIterator &>(dep).ParentVer().DependsList();
+	  !found_conflicts && !possible_conflict.end(); ++possible_conflict)
+	{
+	  if(possible_conflict->Type == pkgCache::Dep::Conflicts &&
+	     (possible_conflict->CompareOp & ~pkgCache::Dep::Or) == pkgCache::Dep::NoOp &&
+	     possible_conflict.TargetPkg() == target)
+	    found_conflicts = true;
+	}
+
+      if(!found_conflicts)
+	return false;
+
+
+      return true;
+    }
+  }
+}
