@@ -1,6 +1,6 @@
 // aptcache.cc
 //
-//  Copyright 1999-2007 Daniel Burrows
+//  Copyright 1999-2008 Daniel Burrows
 //
 //  This program is free software; you can redistribute it and/or modify
 //  it under the terms of the GNU General Public License as published by
@@ -48,10 +48,13 @@
 #include <sys/stat.h>
 
 #include <cwidget/generic/util/eassert.h>
+#include <cwidget/generic/util/ssprintf.h>
 
 #include <sigc++/adaptors/bind.h>
 #include <sigc++/functors/mem_fun.h>
 
+namespace cw = cwidget;
+
 using namespace std;
 
 class aptitudeDepCache::apt_undoer:public undoable
@@ -189,7 +192,7 @@ aptitudeDepCache::aptitudeDepCache(pkgCache *Cache, Policy *Plcy)
 {
   // When the "install recommended packages" flag changes, collect garbage.
 #if 0
-  aptcfg->connect(PACKAGE "::Recommends-Important",
+  aptcfg->connect("Apt::Install-Recommends",
 		  sigc::bind(sigc::mem_fun(*this,
 					   &pkgDepCache::MarkAndSweep),
 			     (undo_group *) NULL));
@@ -226,6 +229,94 @@ void aptitudeDepCache::set_read_only(bool new_read_only)
   read_only = new_read_only;
 }
 
+namespace
+{
+  // User tag syntax:
+  // 
+  // Tag-List ::= Tag+
+  // Tag      ::= regexp([^[:space:]]) | regexp("([^\\"]|\\.)")
+  bool parse_user_tag(std::string &out,
+		      const char *&start, const char *end,
+		      const std::string &package_name)
+  {
+    const char * const initial_start = start;
+    while(start != end && isspace(*start))
+      ++start;
+
+    if(start == end)
+      return false;
+    else if(*start != '"')
+      {
+	while(start != end && !isspace(*start))
+	  {
+	    out += *start;
+	    ++start;
+	  }
+	return true;
+      }
+    else
+      {
+	++start;
+	while(start != end && *start != '"')
+	  {
+	    if(*start == '\\')
+	      {
+		++start;
+		if(start == end)
+		  return _error->Error(_("Error parsing a user-tag for the package %s: unexpected end-of-line following %s."),
+				       package_name.c_str(),
+				       (std::string("\"") + std::string(initial_start, start)).c_str());
+		else
+		  {
+		    out += *start;
+		    ++start;
+		  }
+	      }
+	    else
+	      {
+		out += *start;
+		++start;
+	      }
+	  }
+
+	if(start == end)
+	  return _error->Error(_("Unterminated '\"' in the user-tags list of the package %s."),
+			       package_name.c_str());
+	else
+	  {
+	    ++start;
+	    return true;
+	  }
+      }
+  }
+}
+
+void aptitudeDepCache::parse_user_tags(std::set<user_tag> &tags,
+				       const char *&start, const char *end,
+				       const std::string &package_name)
+{
+  while(start != end)
+    {
+      while(start != end && isspace(*start))
+	++start;
+
+      std::string tag;
+      parse_user_tag(tag, start, end, package_name);
+
+      typedef std::map<std::string, user_tag_reference>::const_iterator
+	user_tags_index_iterator;
+      user_tags_index_iterator found = user_tags_index.find(tag);
+      if(found == user_tags_index.end())
+	{
+	  user_tag_reference loc(user_tags.size());
+	  user_tags.push_back(tag);
+	  std::pair<user_tags_index_iterator, bool> tmp(user_tags_index.insert(std::make_pair(tag, loc)));
+	  found = tmp.first;
+	}
+      tags.insert(user_tag(found->second));
+    }
+}
+
 bool aptitudeDepCache::build_selection_list(OpProgress &Prog, bool WithLock,
 					    bool do_initselections,
 					    const char *status_fname)
@@ -250,10 +341,12 @@ bool aptitudeDepCache::build_selection_list(OpProgress &Prog, bool WithLock,
   // Should this not go under Dir:: ?  I'm not sure..
   delete package_states;
   package_states=new aptitude_state[Head().PackageCount];
+  user_tags.clear();
   for(unsigned int i=0; i<Head().PackageCount; i++)
     {
       package_states[i].new_package=true;
       package_states[i].reinstall=false;
+      package_states[i].user_tags.clear();
       package_states[i].remove_reason=manual;
       package_states[i].selection_state = pkgCache::State::Unknown;
       package_states[i].previously_auto_package = false;
@@ -313,7 +406,8 @@ bool aptitudeDepCache::build_selection_list(OpProgress &Prog, bool WithLock,
       bool do_dselect=aptcfg->FindB(PACKAGE "::Track-Dselect-State", true);
       while(tagfile.Step(section))
 	{
-	  PkgIterator pkg=FindPkg(section.FindS("Package"));
+	  std::string package_name(section.FindS("Package"));
+	  PkgIterator pkg = FindPkg(package_name);
 	  if(!pkg.end() && !pkg.VersionList().end())
 	    // Silently ignore unknown packages and packages with no actual
 	    // version.
@@ -358,6 +452,13 @@ bool aptitudeDepCache::build_selection_list(OpProgress &Prog, bool WithLock,
 	      pkg_state.candver=candver;
 	      pkg_state.forbidver=section.FindS("ForbidVer");
 
+	      {
+		const char *start, *end;
+		if(section.Find("User-Tags", start, end))
+		  parse_user_tags(pkg_state.user_tags, start, end,
+				  package_name);
+	      }
+
 	      if(do_dselect && pkg->SelectedState != last_dselect_state)
 		{
 		  MarkFromDselect(pkg);
@@ -502,6 +603,8 @@ void aptitudeDepCache::mark_all_upgradable(bool with_autoinst,
       return;
     }
 
+  pre_package_state_changed();
+
   action_group group(*this, undo);
 
   for(int iter=0; iter==0 || (iter==1 && with_autoinst); ++iter)
@@ -580,7 +683,14 @@ bool aptitudeDepCache::save_selection_list(OpProgress &prog,
   if(lock==-1 && !status_fname)
     return true;
 
-  writeStateFile(&prog);
+  // Don't write the global apt state file if we're not writing our
+  // own global state.  TODO: this means that su-to-root will lose
+  // automatic states! (but we couldn't do anything about it anyway
+  // because we're not root and so can't write the global state file)
+  // Ow, that sucks.  The solution will be to write the apt states to
+  // a separate file.
+  if(status_fname == NULL)
+    writeStateFile(&prog);
 
   string statefile=_config->FindDir("Dir::Aptitude::state", STATEDIR)+"pkgstates";
 
@@ -607,8 +717,6 @@ bool aptitudeDepCache::save_selection_list(OpProgress &prog,
 	  {
 	    StateCache &state=(*this)[i];
 	    aptitude_state &estate=get_ext_state(i);
-	    char buf[400];
-	    int len;
 
 	    string forbidstr=!estate.forbidver.empty()
 	      ? "ForbidVer: "+estate.forbidver+"\n":"";
@@ -629,28 +737,75 @@ bool aptitudeDepCache::save_selection_list(OpProgress &prog,
 		GetCandidateVer(i).VerStr() != estate.candver))
 	      tailstr = "Version: " + estate.candver + "\n";
 
-	    len=snprintf(buf,
-			 400,
-			 "Package: %s\nUnseen: %s\nState: %i\nDselect-State: %i\nRemove-Reason: %i\n%s%s%s%s\n",
-			 i.Name(),
-			 estate.new_package?"yes":"no",
-			 estate.selection_state,
-			 i->SelectedState,
-			 estate.remove_reason,
-			 upgradestr.c_str(),
-			 autostr.c_str(),
-			 forbidstr.c_str(),
-			 tailstr.c_str());
-	    if(len>=399)
+	    // Build the list of usertags for this package.
+	    std::string user_tags;
+	    if(!estate.user_tags.empty())
 	      {
-		_error->Error(_("Internal buffer overflow on package \"%s\" while writing state file"), i.Name());
-		newstate.Close();
+		user_tags = "User-Tags: ";
 
-		if(!status_fname)
-		  unlink((statefile+".new").c_str());
-		return false;
+		// Put the usertags in sorted order so we get
+		// predictable outputs.
+		std::vector<std::string> tmp;
+		tmp.reserve(estate.user_tags.size());
+
+		for(std::set<user_tag>::const_iterator it
+		      = estate.user_tags.begin(); it != estate.user_tags.end(); ++it)
+		  tmp.push_back(deref_user_tag(*it));
+
+		std::sort(tmp.begin(), tmp.end());
+
+		bool first = true;
+		// Append user tags to the field, using double-quotes
+		// if the tag contains spaces or double-quotes.
+		for(std::vector<std::string>::const_iterator it = tmp.begin();
+		    it != tmp.end(); ++it)
+		  {
+		    if(first)
+		      first = false;
+		    else
+		      user_tags.push_back(' ');
+
+		    if(it->find_first_of(" \"\\") != std::string::npos)
+		      {
+			user_tags.push_back('"');
+			for(std::string::const_iterator tag_it = it->begin();
+			    tag_it != it->end(); ++tag_it)
+			  {
+			    switch(*tag_it)
+			      {
+			      case '"':
+			      case '\\':
+				user_tags.push_back('\\');
+				user_tags.push_back(*tag_it);
+				break;
+			      default:
+				user_tags.push_back(*tag_it);
+				break;
+			      }
+			  }
+			user_tags.push_back('"');
+		      }
+		    else
+		      user_tags.insert(user_tags.size(), *it);
+		  }
+
+		user_tags.push_back('\n');
 	      }
-	    if(newstate.Failed() || !newstate.Write(buf, len))
+
+	    using cw::util::ssprintf;
+	    std::string line(ssprintf("Package: %s\nUnseen: %s\nState: %i\nDselect-State: %i\nRemove-Reason: %i\n%s%s%s%s%s\n",
+				      i.Name(),
+				      estate.new_package?"yes":"no",
+				      estate.selection_state,
+				      i->SelectedState,
+				      estate.remove_reason,
+				      upgradestr.c_str(),
+				      autostr.c_str(),
+				      forbidstr.c_str(),
+				      user_tags.c_str(),
+				      tailstr.c_str()));
+
+	    if(newstate.Failed() || !newstate.Write(line.c_str(), line.size()))
 	      {
 		_error->Error(_("Couldn't write state file"));
 		newstate.Close();
@@ -801,6 +956,18 @@ void aptitudeDepCache::cleanup_after_change(undo_group *undo, bool alter_stickie
 	 package_states[pkg->ID].remove_reason!=backup_state.AptitudeState[pkg->ID].remove_reason ||
 	 package_states[pkg->ID].forbidver!=backup_state.AptitudeState[pkg->ID].forbidver)
 	{
+	  // Technically this could be invoked only when we're really
+	  // about to change a package's state, but placing it here
+	  // should avoid signalling changes unnecessarily while still
+	  // signalling a change whenever something changed.
+	  //
+	  // You could argue that this invocation of
+	  // pre_package_state_changed makes all other unnecessary,
+	  // but I think they should be left in for safety's sake.
+	  // Forgetting to call pre_package_state_changed can lead to
+	  // hard to track down bugs like #432411.
+	  pre_package_state_changed();
+
 	  int n=pkg->ID;
 	  n=n;
 	  char curM=PkgState[pkg->ID].Mode;
@@ -1130,6 +1297,112 @@ void aptitudeDepCache::mark_auto_installed(const PkgIterator &Pkg,
   MarkAuto(Pkg, set_auto);
 }
 
+// Undoers for the tag manipulators below.
+namespace
+{
+  class attach_user_tag_undoer : public undoable
+  {
+    aptitudeDepCache *parent;
+    pkgCache::PkgIterator pkg;
+    std::string tag;
+
+  public:
+    attach_user_tag_undoer(aptitudeDepCache *_parent,
+		      const pkgCache::PkgIterator &_pkg,
+		      const std::string &_tag)
+      : parent(_parent), pkg(_pkg), tag(_tag)
+    {
+    }
+
+    void undo()
+    {
+      parent->detach_user_tag(pkg, tag, NULL);
+    }
+  };
+
+  class detach_user_tag_undoer : public undoable
+  {
+    aptitudeDepCache *parent;
+    pkgCache::PkgIterator pkg;
+    std::string tag;
+
+  public:
+    detach_user_tag_undoer(aptitudeDepCache *_parent,
+			   const pkgCache::PkgIterator &_pkg,
+			   const std::string &_tag)
+      : parent(_parent), pkg(_pkg), tag(_tag)
+    {
+    }
+
+    void undo()
+    {
+      parent->attach_user_tag(pkg, tag, NULL);
+    }
+  };
+}
+
+void aptitudeDepCache::attach_user_tag(const PkgIterator &pkg,
+				       const std::string &tag,
+				       undo_group *undo)
+{
+  if(read_only && !read_only_permission())
+    {
+      if(group_level == 0)
+	read_only_fail();
+      return;
+    }
+
+  // Find the tag in our cache or add it.
+  typedef std::map<std::string, user_tag_reference>::const_iterator index_ref;
+  index_ref found = user_tags_index.find(tag);
+
+  if(found == user_tags_index.end())
+    {
+      user_tag_reference loc = user_tags.size();
+      user_tags.push_back(tag);
+      std::pair<index_ref, bool> tmp(user_tags_index.insert(std::make_pair(tag, loc)));
+      found = tmp.first;
+    }
+
+  std::pair<std::set<user_tag>::const_iterator, bool> insert_result =
+    get_ext_state(pkg).user_tags.insert(user_tag(found->second));
+
+  if(insert_result.second)
+    {
+      dirty = true;
+      if(undo != NULL)
+	undo->add_item(new attach_user_tag_undoer(this, pkg, tag));
+    }
+}
+
+void aptitudeDepCache::detach_user_tag(const PkgIterator &pkg,
+				       const std::string &tag,
+				       undo_group *undo)
+{
+  if(read_only && !read_only_permission())
+    {
+      if(group_level == 0)
+	read_only_fail();
+      return;
+    }
+
+  std::map<std::string, user_tag_reference>::const_iterator found =
+    user_tags_index.find(tag);
+
+  if(found == user_tags_index.end())
+    return;
+
+  std::set<user_tag>::size_type num_erased =
+    get_ext_state(pkg).user_tags.erase(user_tag(found->second));
+
+  if(num_erased > 0)
+    {
+      dirty = true;
+      if(undo != NULL)
+	undo->add_item(new detach_user_tag_undoer(this, pkg, tag));
+    }
+}
+
 bool aptitudeDepCache::all_upgrade(bool with_autoinst, undo_group *undo)
 {
   if(read_only && !read_only_permission())
@@ -1317,6 +1590,7 @@ void aptitudeDepCache::sweep()
 	      // not previously being deleted.
 	      if(!PkgState[pkg->ID].Delete())
 		{
+		  pre_package_state_changed();
 		  MarkDelete(pkg, purge_unused);
 		  package_states[pkg->ID].selection_state =
 		    (purge_unused ? pkgCache::State::Purge : pkgCache::State::DeInstall);
@@ -1334,11 +1608,13 @@ void aptitudeDepCache::sweep()
 		}
 	      else
 		package_states[pkg->ID].selection_state = pkgCache::State::Install;
+	      pre_package_state_changed();
 	      MarkKeep(pkg, false, false);
 	    }
 	}
       else if(PkgState[pkg->ID].Delete() && package_states[pkg->ID].remove_reason == unused)
 	{
+	  pre_package_state_changed();
 	  MarkKeep(pkg, false, false);
 	}
     }
@@ -1410,9 +1686,12 @@ void aptitudeDepCache::restore_apt_state(const apt_state_snapshot *snapshot)
   iBadCount=snapshot->iBadCount;
 }
 
-void aptitudeDepCache::apply_solution(const generic_solution<aptitude_universe> &sol,
+void aptitudeDepCache::apply_solution(const generic_solution<aptitude_universe> &realSol,
 				      undo_group *undo)
 {
+  // Make a local copy so we don't crash when applying the solution.
+  const generic_solution<aptitude_universe> sol(realSol);
+
   if(read_only && !read_only_permission())
     {
       if(group_level == 0)
@@ -1561,7 +1840,7 @@ bool aptitudeDepCache::is_held(const PkgIterator &pkg)
 bool aptitudeDepCache::MarkFollowsRecommends()
 {
   return pkgDepCache::MarkFollowsRecommends() ||
-    aptcfg->FindB(PACKAGE "::Recommends-Important", true) ||
+    aptcfg->FindB("Apt::Install-Recommends", true) ||
     aptcfg->FindB(PACKAGE "::Keep-Recommends", false);
 }
 
@@ -1578,7 +1857,7 @@ class AptitudeInRootSetFunc : public pkgDepCache::InRootSetFunc
   aptitudeDepCache &cache;
 
   /** A matcher if one could be created; otherwise NULL. */
-  pkg_matcher *m;
+  aptitude::matching::pkg_matcher *m;
 
   /** \b true if the package was successfully constructed. */
   bool constructedSuccessfully;
@@ -1600,7 +1879,7 @@ public:
       constructedSuccessfully = true;
     else
       {
-	m = parse_pattern(matchterm);
+	m = aptitude::matching::parse_pattern(matchterm);
 	if(m != NULL)
 	  constructedSuccessfully = true;
       }
@@ -1617,7 +1896,7 @@ public:
   bool InRootSet(const pkgCache::PkgIterator &pkg)
   {
     pkgRecords &records(cache.get_records());
-    if(m != NULL && m->matches(pkg, cache, records))
+    if(m != NULL && aptitude::matching::apply_matcher(m, pkg, cache, records))
       return true;
     else
       return chain != NULL && chain->InRootSet(pkg);
@@ -1644,3 +1923,22 @@ pkgDepCache::InRootSetFunc *aptitudeDepCache::GetRootSetFunc()
       return superFunc;
     }
 }
+
+bool aptitudeDepCache::AutoInstOk(const PkgIterator &pkg,
+				  const VerIterator &ver,
+				  const DepIterator &dep)
+{
+  const aptitude_state &estate = get_ext_state(pkg);
+
+  // Don't break holds.
+  if(estate.selection_state == pkgCache::State::Hold &&
+     ver != pkg.CurrentVer())
+    return false;
+
+  // Don't install forbidden versions.
+  if(!ver.end() && estate.forbidver == ver.VerStr())
+    return false;
+
+  // Everything else is A-OK.
+  return true;
+}
