@@ -1,6 +1,6 @@
 // resolver_manager.cc
 //
-//   Copyright (C) 2005, 2007 Daniel Burrows
+//   Copyright (C) 2005, 2007-2008 Daniel Burrows
 //
 //   This program is free software; you can redistribute it and/or
 //   modify it under the terms of the GNU General Public License as
@@ -23,12 +23,131 @@
 #include "aptitude_resolver.h"
 #include "aptitude_resolver_universe.h"
 #include "config_signal.h"
+#include "dump_packages.h"
 
 #include <generic/problemresolver/problemresolver.h>
+#include <generic/util/temp.h>
 #include <generic/util/undo.h>
 
+#include <apt-pkg/error.h>
+#include <apt-pkg/strutl.h>
+
 #include <sigc++/functors/mem_fun.h>
 
+#include <fstream>
+
+#include <sys/types.h>
+#include <sys/wait.h>
+
+class resolver_manager::resolver_interaction
+{
+public:
+  /** \brief The type tag of a resolver interaction. */
+  enum tag
+    {
+      reject_version,
+      unreject_version,
+      mandate_version,
+      unmandate_version,
+      harden_dep,
+      unharden_dep,
+      approve_broken_dep,
+      unapprove_broken_dep,
+      undo
+    };
+
+private:
+  tag type;
+
+  aptitude_resolver_version version;
+  aptitude_resolver_dep dep;
+
+  resolver_interaction(tag _type,
+		       const aptitude_resolver_version &_version,
+		       const aptitude_resolver_dep &_dep)
+    : type(_type), version(_version), dep(_dep)
+  {
+  }
+
+public:
+  static resolver_interaction RejectVersion(const aptitude_resolver_version &version)
+  {
+    return resolver_interaction(reject_version, version,
+				aptitude_resolver_dep());
+  }
+
+  static resolver_interaction UnRejectVersion(const aptitude_resolver_version &version)
+  {
+    return resolver_interaction(unreject_version, version,
+				aptitude_resolver_dep());
+  }
+
+  static resolver_interaction MandateVersion(const aptitude_resolver_version &version)
+  {
+    return resolver_interaction(mandate_version, version,
+				aptitude_resolver_dep());
+  }
+
+  static resolver_interaction UnMandateVersion(const aptitude_resolver_version &version)
+  {
+    return resolver_interaction(unmandate_version, version,
+				aptitude_resolver_dep());
+  }
+
+  static resolver_interaction HardenDep(const aptitude_resolver_dep &dep)
+  {
+    return resolver_interaction(harden_dep,
+				aptitude_resolver_version(),
+				dep);
+  }
+
+  static resolver_interaction UnHardenDep(const aptitude_resolver_dep &dep)
+  {
+    return resolver_interaction(unharden_dep,
+				aptitude_resolver_version(),
+				dep);
+  }
+
+  static resolver_interaction ApproveBrokenDep(const aptitude_resolver_dep &dep)
+  {
+    return resolver_interaction(approve_broken_dep,
+				aptitude_resolver_version(),
+				dep);
+  }
+
+  static resolver_interaction UnApproveBrokenDep(const aptitude_resolver_dep &dep)
+  {
+    return resolver_interaction(unapprove_broken_dep,
+				aptitude_resolver_version(),
+				dep);
+  }
+
+  static resolver_interaction Undo()
+  {
+    return resolver_interaction(undo,
+				aptitude_resolver_version(),
+				aptitude_resolver_dep());
+  }
+
+  tag get_type() const { return type; }
+  const aptitude_resolver_version &get_version() const
+  {
+    eassert(!version.get_pkg().end());
+    return version;
+  }
+  const aptitude_resolver_dep &get_dep() const
+  {
+    eassert(!dep.get_dep().end());
+    return dep;
+  }
+};
+
+resolver_manager::solution_information::~solution_information()
+{
+  delete interactions;
+  delete solution;
+}
+
 // NB: we need a recursive mutex because some routines can be called
 // either by other routines of the class (already have a mutex lock)
 // or by the user (don't have a mutex lock); I could sidestep this
@@ -38,6 +157,7 @@ resolver_manager::resolver_manager(aptitudeDepCache *_cache)
   :cache(_cache),
    resolver(NULL),
    undos(new undo_list),
+   ticks_since_last_solution(0),
    solution_search_aborted(false),
    selected_solution(0),
    background_thread_killed(false),
@@ -51,7 +171,7 @@ resolver_manager::resolver_manager(aptitudeDepCache *_cache)
   cache->pre_package_state_changed.connect(sigc::mem_fun(this, &resolver_manager::discard_resolver));
   cache->package_state_changed.connect(sigc::mem_fun(this, &resolver_manager::maybe_create_resolver));
 
-  aptcfg->connect(PACKAGE "::Recommends-Important",
+  aptcfg->connect("Apt::Install-Recommends",
 		  sigc::mem_fun(this,
 				&resolver_manager::discard_resolver));
 
@@ -68,12 +188,21 @@ resolver_manager::~resolver_manager()
 
   kill_background_thread();
 
-  for(unsigned int i = 0; i < solutions.size(); ++i)
-    delete solutions[i];
+  for(std::vector<const solution_information *>::const_iterator it =
+	solutions.begin(); it != solutions.end(); ++it)
+    {
+      delete *it;
+    }
 
   delete undos;
 }
 
+void resolver_manager::reset_resolver()
+{
+  discard_resolver();
+  maybe_create_resolver();
+}
+
 resolver_manager::background_continuation::~background_continuation()
 {
 }
@@ -132,6 +261,196 @@ public:
   }
 };
 
+void resolver_manager::write_test_control_file(const std::string &outDir,
+					       const std::set<aptitude_resolver_package> &visited_packages,
+					       int solution_number)
+{
+  static const char * const strBadChars = "\\\"";
+
+  std::string control_filename = outDir + "/APTITUDE.TRACE";
+  std::ofstream control_file(control_filename.c_str());
+  if(!control_file)
+    return; // Should we output an error here?
+
+  // The state files are written out in make_truncated_state_copy.
+  // Here we just indicate the actual installs / removals that are
+  // queued up.
+
+  control_file << "initial {" << std::endl;
+  for(std::set<aptitude_resolver_package>::const_iterator
+	it = visited_packages.begin();
+      it != visited_packages.end();
+      ++it)
+    {
+      const aptitude_resolver_package &p(*it);
+      if(visited_packages.find(p) == visited_packages.end())
+	continue;
+
+      pkg_action_state action = find_pkg_state(p.get_pkg(), *apt_cache_file);
+      std::string actionstr;
+      switch(action)
+	{
+	case pkg_unused_remove:
+	case pkg_auto_remove:
+	case pkg_remove:
+	  actionstr = "remove";
+	  break;
+
+	case pkg_auto_install:
+	case pkg_downgrade:
+	case pkg_install:
+	case pkg_upgrade:
+	  actionstr = std::string("install ") +
+	    (*apt_cache_file)[p.get_pkg()].InstVerIter(*apt_cache_file).VerStr();
+	  break;
+
+	default:
+	  break;
+	}
+
+      if(!actionstr.empty())
+	control_file << "  " << actionstr << " \"" << QuoteString(p.get_name(), strBadChars) << "\"" << std::endl;;
+    }
+
+  control_file << "}" << std::endl;
+  control_file << std::endl;
+
+  {
+    cwidget::threads::mutex::lock sol_l(solutions_mutex);
+
+    for(std::vector<const solution_information *>::const_iterator infIt =
+	  solutions.begin();
+	infIt != solutions.end() && infIt - solutions.begin() <= solution_number;
+	++infIt)
+      {
+	control_file << "test {" << std::endl;
+
+	const solution_information &inf = **infIt;
+	const std::vector<resolver_interaction> &acts =
+	  *inf.get_interactions();
+	const generic_solution<aptitude_universe> &sol = *inf.get_solution();
+
+	for(std::vector<resolver_interaction>::const_iterator actIt =
+	      acts.begin(); actIt != acts.end(); ++actIt)
+	  {
+	    switch(actIt->get_type())
+	      {
+	      case resolver_interaction::reject_version:
+		control_file << "reject " << actIt->get_version() << std::endl;
+		break;
+	      case resolver_interaction::unreject_version:
+		control_file << "unreject " << actIt->get_version() << std::endl;
+		break;
+	      case resolver_interaction::mandate_version:
+		control_file << "mandate " << actIt->get_version() << std::endl;
+		break;
+	      case resolver_interaction::unmandate_version:
+		control_file << "unmandate " << actIt->get_version() << std::endl;
+		break;
+	      case resolver_interaction::harden_dep:
+		control_file << "harden " << actIt->get_dep() << std::endl;
+		break;
+	      case resolver_interaction::unharden_dep:
+		control_file << "unharden " << actIt->get_dep() << std::endl;
+		break;
+	      case resolver_interaction::approve_broken_dep:
+		control_file << "approve_broken " << actIt->get_dep() << std::endl;
+		break;
+	      case resolver_interaction::unapprove_broken_dep:
+		control_file << "unapprove_broken " << actIt->get_dep() << std::endl;
+		break;
+	      case resolver_interaction::undo:
+		control_file << "undo" << std::endl;
+		break;
+	      }
+	  }
+
+	control_file << "  expect " << inf.get_ticks() << " {" << std::endl;
+
+	typedef generic_solution<aptitude_universe>::action action;
+	typedef aptitude_resolver_package package;
+	const imm::map<package, action> &actions = sol.get_actions();
+	for(imm::map<package, action>::const_iterator solActIt =
+	      actions.begin(); solActIt != actions.end(); ++solActIt)
+	  {
+	    control_file << "    \""
+			 << QuoteString(solActIt->first.get_name(), strBadChars)
+			 << "\" \""
+			 << QuoteString(solActIt->second.ver.get_name(), strBadChars)
+			 << "\""
+			 << std::endl;
+	  }
+	control_file << "  }" << std::endl;
+
+	control_file << "}" << std::endl;
+      }
+  }
+}
+
+void resolver_manager::dump_visited_packages(const std::set<aptitude_resolver_package> &visited_packages,
+					     int solution_number)
+{
+  std::string trace_dir;
+  temp::dir temp_trace_dir;
+  if(!resolver_trace_dir.empty())
+    trace_dir = resolver_trace_dir;
+  else if(!resolver_trace_file.empty())
+    {
+      temp_trace_dir = temp::dir("aptitude-trace-dump", true);
+      trace_dir = temp_trace_dir.get_name();
+    }
+  else
+    return;
+
+  std::set<pkgCache::PkgIterator> packages;
+  for(std::set<aptitude_resolver_package>::const_iterator it = visited_packages.begin();
+      it != visited_packages.end(); ++it)
+    {
+      packages.insert((*it).get_pkg());
+    }
+
+  aptitude::apt::make_truncated_state_copy(trace_dir, packages);
+  write_test_control_file(trace_dir, visited_packages, solution_number);
+
+  if(!resolver_trace_file.empty())
+    {
+      int pid = fork();
+      if(pid == -1)
+	_error->Errno("fork", "Unable to run \"tar\" to create the trace file.");
+      else if(pid != 0)
+	{
+	  int status;
+	  while(waitpid(pid, &status, 0) != pid ||
+		(!WIFEXITED(status) &&
+		 !WIFSIGNALED(status)))
+	    ;
+
+	  if(WIFSIGNALED(status))
+	    _error->Error("Unable to create the output file: child killed by signal %d.", WTERMSIG(status));
+	  else if(WEXITSTATUS(status) != 0)
+	    _error->Error("Unable to create the output file: child exited with status %d.", WEXITSTATUS(status));
+
+	  // Whee, we win!
+	}
+      else
+	{
+	  // Should I close open fds here?
+
+	  // Should I allow tar to be found on the path?
+	  execl("/bin/tar", "/bin/tar", "czf",
+		resolver_trace_file.c_str(),
+		"-C",
+		trace_dir.c_str(),
+		".",
+		NULL);
+
+	  // TODO: communicate to the parent that we couldn't find
+	  // tar?
+	  exit(1);
+	}
+    }
+}
+
 // This assumes that background_resolver_active is empty when it
 // starts (see restart_background_resolver)
 //
@@ -140,6 +459,8 @@ public:
 // interactively)
 void resolver_manager::background_thread_execution()
 {
+  std::set<aptitude_resolver_package> visited_packages;
+
   cwidget::threads::mutex::lock l(background_control_mutex);
   set_when_destroyed<bool> cancel_set_running(background_thread_running, false);
 
@@ -160,14 +481,18 @@ void resolver_manager::background_thread_execution()
 
       try
 	{
-	  aptitude_resolver::solution *sol = do_get_solution(job.max_steps,
-							     job.sol_num);
+	  const aptitude_resolver::solution *sol =
+	    do_get_solution(job.max_steps,
+			    job.sol_num,
+			    visited_packages);
 
 	  // Set the state variable BEFORE exiting the resolver; this
 	  // is done so that if there are no more jobs, the foreground
 	  // thread sees that we're out of the resolver when it
 	  // examines the solution.
 	  l.acquire();
+	  dump_visited_packages(visited_packages,
+				job.sol_num);
 	  background_thread_in_resolver = false;
 	  background_resolver_cond.wake_all();
 	  l.release();
@@ -178,6 +503,8 @@ void resolver_manager::background_thread_execution()
 	{
 	  // Put it back into the pot.
 	  l.acquire();
+	  dump_visited_packages(visited_packages,
+				job.sol_num);
 	  background_thread_in_resolver = false;
 	  background_resolver_cond.wake_all();
 	  pending_jobs.push(job);
@@ -189,6 +516,8 @@ void resolver_manager::background_thread_execution()
       catch(NoMoreSolutions)
 	{
 	  l.acquire();
+	  dump_visited_packages(visited_packages,
+				job.sol_num);
 	  background_thread_in_resolver = false;
 	  background_resolver_cond.wake_all();
 	  l.release();
@@ -198,6 +527,8 @@ void resolver_manager::background_thread_execution()
       catch(NoMoreTime)
 	{
 	  l.acquire();
+	  dump_visited_packages(visited_packages,
+				job.sol_num);
 	  background_thread_in_resolver = false;
 	  background_resolver_cond.wake_all();
 	  l.release();
@@ -206,6 +537,8 @@ void resolver_manager::background_thread_execution()
 	}
       catch(cwidget::util::Exception &e)
 	{
+	  dump_visited_packages(visited_packages,
+				job.sol_num);
 	  job.k->aborted(e);
 	}
 
@@ -318,7 +651,14 @@ void resolver_manager::maybe_create_resolver()
   cwidget::threads::mutex::lock l(mutex);
 
   if(resolver == NULL && cache->BrokenCount() > 0)
-    create_resolver();
+    {
+      {
+	cwidget::threads::mutex::lock l(background_control_mutex);
+	resolver_trace_dir = aptcfg->Find(PACKAGE "::ProblemResolver::Trace-Directory", "");
+	resolver_trace_file = aptcfg->Find(PACKAGE "::ProblemResolver::Trace-File", "");
+      }
+      create_resolver();
+    }
 
   // Always signal a state change: we are signalling for the whole
   // discard/create pair, and even if we didn't create a new resolver
@@ -343,6 +683,13 @@ void resolver_manager::discard_resolver()
 
   {
     cwidget::threads::mutex::lock l2(solutions_mutex);
+    actions_since_last_solution.clear();
+    ticks_since_last_solution = 0;
+
+    for(std::vector<const solution_information *>::const_iterator it =
+	  solutions.begin(); it != solutions.end(); ++it)
+      delete *it;
+
     solutions.clear();
     solution_search_aborted = false;
     solution_search_abort_msg.clear();
@@ -398,7 +745,10 @@ void resolver_manager::create_resolver()
 			      aptcfg->FindI(PACKAGE "::ProblemResolver::UpgradeScore", 0),
 			      aptcfg->FindI(PACKAGE "::ProblemResolver::NonDefaultScore", -40),
 			      aptcfg->FindI(PACKAGE "::ProblemResolver::EssentialRemoveScore", -100000),
-			      aptcfg->FindI(PACKAGE "::ProblemResolver::BreakHoldScore", -300));
+			      aptcfg->FindI(PACKAGE "::ProblemResolver::FullReplacementScore", 500),
+			      aptcfg->FindI(PACKAGE "::ProblemResolver::UndoFullReplacementScore", -500),
+			      aptcfg->FindI(PACKAGE "::ProblemResolver::BreakHoldScore", -300),
+			      aptcfg->FindB(PACKAGE "::ProblemResolver::Allow-Break-Holds", false));
 
   resolver->add_priority_scores(aptcfg->FindI(PACKAGE "::ProblemResolver::ImportantScore", 5),
 				aptcfg->FindI(PACKAGE "::ProblemResolver::RequiredScore", 4),
@@ -413,6 +763,12 @@ void resolver_manager::create_resolver()
   }
 }
 
+void resolver_manager::set_resolver_trace_dir(const std::string &path)
+{
+  cwidget::threads::mutex::lock l(background_control_mutex);
+  resolver_trace_dir = path;
+}
+
 void resolver_manager::set_debug(bool activate)
 {
   cwidget::threads::mutex::lock l(mutex);
@@ -522,11 +878,13 @@ resolver_manager::state resolver_manager::state_snapshot()
   return rval;
 }
 
-aptitude_resolver::solution *resolver_manager::do_get_solution(int max_steps, unsigned int solution_num)
+const aptitude_resolver::solution *
+resolver_manager::do_get_solution(int max_steps, unsigned int solution_num,
+				  std::set<aptitude_resolver_package> &visited_packages)
 {
   cwidget::threads::mutex::lock sol_l(solutions_mutex);
   if(solution_num < solutions.size())
-    return solutions[solution_num];
+    return solutions[solution_num]->get_solution();
 
   while(solution_num >= solutions.size())
     {
@@ -534,14 +892,28 @@ aptitude_resolver::solution *resolver_manager::do_get_solution(int max_steps, un
 
       try
 	{
-	  generic_solution<aptitude_universe> sol = resolver->find_next_solution(max_steps);
+	  generic_solution<aptitude_universe> sol = resolver->find_next_solution(max_steps, &visited_packages);
 
 	  sol_l.acquire();
-	  solutions.push_back(new aptitude_resolver::solution(sol.clone()));
+
+	  bool is_keep_all_solution =
+	    (sol.get_actions() == resolver->get_keep_all_solution());
+
+	  solutions.push_back(new solution_information(new std::vector<resolver_interaction>(actions_since_last_solution),
+						       ticks_since_last_solution + max_steps,
+						       new aptitude_resolver::solution(sol.clone()),
+						       is_keep_all_solution));
+	  actions_since_last_solution.clear();
 	  sol_l.release();
 	}
+      catch(const InterruptedException &e)
+	{
+	  ticks_since_last_solution += e.get_steps();
+	  throw e;
+	}
       catch(NoMoreTime)
 	{
+	  ticks_since_last_solution += max_steps;
 	  throw NoMoreTime();
 	}
       catch(NoMoreSolutions)
@@ -557,7 +929,7 @@ aptitude_resolver::solution *resolver_manager::do_get_solution(int max_steps, un
 	}
     }
 
-  return solutions[solution_num];
+  return solutions[solution_num]->get_solution();
 }
 
 /** A continuation that works by either placing \b true in the Boolean
@@ -653,7 +1025,7 @@ const aptitude_resolver::solution &resolver_manager::get_solution(unsigned int s
   {
     cwidget::threads::mutex::lock l2(solutions_mutex);
     if(solution_num < solutions.size())
-      return *solutions[solution_num];
+      return *solutions[solution_num]->get_solution();
   }
 
 
@@ -682,6 +1054,21 @@ const aptitude_resolver::solution &resolver_manager::get_solution(unsigned int s
   return *sol;
 }
 
+bool resolver_manager::get_is_keep_all_solution(unsigned int solution_num,
+						int max_steps)
+{
+  cwidget::threads::mutex::lock l(mutex);
+
+  get_solution(solution_num, max_steps);
+
+  {
+    cwidget::threads::mutex::lock l2(solutions_mutex);
+    eassert(solution_num < solutions.size());
+
+    return solutions[solution_num]->get_is_keep_all_solution();;
+  }
+}
+
 void resolver_manager::get_solution_background(unsigned int solution_num,
 					       int max_steps,
 					       background_continuation *k)
@@ -697,7 +1084,7 @@ void resolver_manager::get_solution_background(unsigned int solution_num,
   cwidget::threads::mutex::lock sol_l(solutions_mutex);
   if(solution_num < solutions.size())
     {
-      generic_solution<aptitude_universe> *sol = solutions[solution_num];
+      const generic_solution<aptitude_universe> *sol = solutions[solution_num]->get_solution();
       sol_l.release();
 
       k->success(*sol);
@@ -805,7 +1192,8 @@ bool resolver_manager::get_solution_background_blocking(unsigned int solution_nu
 
 template<typename T>
 void resolver_manager::resolver_manipulation(const T &t,
-					     void (generic_problem_resolver<aptitude_universe>::*action)(const T &, undo_group *))
+					     void (generic_problem_resolver<aptitude_universe>::*action)(const T &, undo_group *),
+					     const resolver_interaction &act)
 {
   cwidget::threads::mutex::lock l(mutex);
   background_suspender bs(*this);
@@ -817,6 +1205,39 @@ void resolver_manager::resolver_manipulation(const T &t,
   else
     undos->add_item(undo);
 
+  actions_since_last_solution.push_back(act);
+
+  l.release();
+  bs.unsuspend();
+  state_changed();
+}
+
+void resolver_manager::reject_break_holds()
+{
+  cwidget::threads::mutex::lock l(mutex);
+  background_suspender bs(*this);
+
+  std::auto_ptr<undo_group> undo(new undo_group);
+
+  for(aptitude_universe::package_iterator pi = resolver->get_universe().packages_begin();
+      !pi.end(); ++pi)
+    {
+      const aptitude_universe::package p = *pi;
+
+      for(aptitude_universe::package::version_iterator vi = p.versions_begin(); !vi.end(); ++vi)
+	{
+	  const aptitude_universe::version v = *vi;
+	  if(resolver->is_break_hold(v))
+	    {
+	      actions_since_last_solution.push_back(resolver_interaction::RejectVersion(v));
+	      reject_version(v);
+	    }
+	}
+    }
+
+  if(!undo->empty())
+    undos->add_item(undo.release());
+
   l.release();
   bs.unsuspend();
   state_changed();
@@ -824,12 +1245,14 @@ void resolver_manager::resolver_manipulation(const T &t,
 
 void resolver_manager::reject_version(const aptitude_resolver_version &ver)
 {
-  resolver_manipulation(ver, &aptitude_resolver::reject_version);
+  resolver_manipulation(ver, &aptitude_resolver::reject_version,
+			resolver_interaction::RejectVersion(ver));
 }
 
 void resolver_manager::unreject_version(const aptitude_resolver_version &ver)
 {
-  resolver_manipulation(ver, &aptitude_resolver::unreject_version);
+  resolver_manipulation(ver, &aptitude_resolver::unreject_version,
+			resolver_interaction::UnRejectVersion(ver));
 }
 
 bool resolver_manager::is_rejected(const aptitude_resolver_version &ver)
@@ -842,12 +1265,14 @@ bool resolver_manager::is_rejected(const aptitude_resolver_version &ver)
 
 void resolver_manager::mandate_version(const aptitude_resolver_version &ver)
 {
-  resolver_manipulation(ver, &aptitude_resolver::mandate_version);
+  resolver_manipulation(ver, &aptitude_resolver::mandate_version,
+			resolver_interaction::MandateVersion(ver));
 }
 
 void resolver_manager::unmandate_version(const aptitude_resolver_version &ver)
 {
-  resolver_manipulation(ver, &aptitude_resolver::unmandate_version);
+  resolver_manipulation(ver, &aptitude_resolver::unmandate_version,
+			resolver_interaction::UnMandateVersion(ver));
 }
 
 bool resolver_manager::is_mandatory(const aptitude_resolver_version &ver)
@@ -860,12 +1285,14 @@ bool resolver_manager::is_mandatory(const aptitude_resolver_version &ver)
 
 void resolver_manager::harden_dep(const aptitude_resolver_dep &dep)
 {
-  resolver_manipulation(dep, &aptitude_resolver::harden);
+  resolver_manipulation(dep, &aptitude_resolver::harden,
+			resolver_interaction::HardenDep(dep));
 }
 
 void resolver_manager::unharden_dep(const aptitude_resolver_dep &dep)
 {
-  resolver_manipulation(dep, &aptitude_resolver::unharden);
+  resolver_manipulation(dep, &aptitude_resolver::unharden,
+			resolver_interaction::UnHardenDep(dep));
 }
 
 bool resolver_manager::is_hardened(const aptitude_resolver_dep &dep)
@@ -878,12 +1305,14 @@ bool resolver_manager::is_hardened(const aptitude_resolver_dep &dep)
 
 void resolver_manager::approve_broken_dep(const aptitude_resolver_dep &dep)
 {
-  resolver_manipulation(dep, &aptitude_resolver::approve_break);
+  resolver_manipulation(dep, &aptitude_resolver::approve_break,
+			resolver_interaction::ApproveBrokenDep(dep));
 }
 
 void resolver_manager::unapprove_broken_dep(const aptitude_resolver_dep &dep)
 {
-  resolver_manipulation(dep, &aptitude_resolver::unapprove_break);
+  resolver_manipulation(dep, &aptitude_resolver::unapprove_break,
+			resolver_interaction::UnApproveBrokenDep(dep));
 }
 
 bool resolver_manager::is_approved_broken(const aptitude_resolver_dep &dep)
@@ -911,6 +1340,8 @@ bool resolver_manager::undo()
 
       undos->undo();
 
+      actions_since_last_solution.push_back(resolver_interaction::Undo());
+
       bs.unsuspend();
       l.release();
 
