@@ -1,6 +1,6 @@
 // resolver_manager.h                                -*-c++-*-
 //
-//   Copyright (C) 2005, 2007 Daniel Burrows
+//   Copyright (C) 2005, 2007-2008 Daniel Burrows
 //
 //   This program is free software; you can redistribute it and/or
 //   modify it under the terms of the GNU General Public License as
@@ -17,12 +17,7 @@
 //   the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 //   Boston, MA 02111-1307, USA.
 //
-// A higher-level resolver interface.  This code is responsible for
-// maintaining a list of previously observed solutions, for passing
-// certain actions on to the underlying resolver (protecting users
-// from having to actually import the whole resolver definition), for
-// managing the resolver in the face of cache reloads and resets, and
-// for managing threaded access to the resolver.
+
 
 #ifndef RESOLVER_MANAGER_H
 #define RESOLVER_MANAGER_H
@@ -37,12 +32,27 @@
 #include <sigc++/trackable.h>
 
 #include <queue>
+#include <set>
 #include <vector>
 
-class aptitudeDepCache;
+/** \brief A higher-level resolver interface
+ *
+ * 
+ *  A higher-level resolver interface. This code is responsible for
+ *  maintaining a list of previously observed solutions, for passing
+ *  certain actions on to the underlying resolver (protecting users
+ *  from having to actually import the whole resolver definition), for
+ *  managing the resolver in the face of cache reloads and resets, and
+ *  for managing threaded access to the resolver.
+ * 
+ *  \file resolver_manager.h
+ */
+
 class aptitude_universe;
+class aptitude_resolver_package;
 class aptitude_resolver_version;
 class aptitude_resolver_dep;
+class aptitudeDepCache;
 template<typename PackageUniverse> class generic_solution;
 template<typename PackageUniverse> class generic_problem_resolver;
 class aptitude_resolver;
@@ -139,6 +149,12 @@ public:
   };
 
 private:
+  /** \brief Remembers a single user interaction with the resolver.
+   *
+   *  This is used to save a debug trace.
+   */
+  class resolver_interaction;
+
   /** Information about a single request posted to the background
    *  thread.
    */
@@ -180,10 +196,76 @@ private:
    */
   undo_list *undos;
 
-  /** The solutions generated by this manager since the last change to
-   *  the cache.
+  /** \brief The operations performed by the user since the last
+   *  solution was generated.
+   *
+   *  Used to generate the actions-since-the-last-solution when a new
+   *  solution is generated.  This should always be accessed with the
+   *  background thread suspended.
    */
-  std::vector<generic_solution<aptitude_universe> *> solutions;
+  std::vector<resolver_interaction> actions_since_last_solution;
+
+  /** \brief The number of ticks for which the resolver has been
+   *  run since the last solution.
+   *
+   *  Like actions_since_last_solution, should be accessed with the
+   *  background thread suspended.
+   */
+  int ticks_since_last_solution;
+
+  /** \brief Stores the information needed to reproduce a solution. */
+  class solution_information
+  {
+    const std::vector<resolver_interaction> *interactions;
+    int ticks;
+    const generic_solution<aptitude_universe> *solution;
+    bool is_keep_all_solution;
+
+  public:
+    solution_information(const std::vector<resolver_interaction> *_interactions,
+			 int _ticks,
+			 const generic_solution<aptitude_universe> *_solution,
+			 bool _is_keep_all_solution)
+      : interactions(_interactions), ticks(_ticks), solution(_solution),
+	is_keep_all_solution(_is_keep_all_solution)
+    {
+    }
+
+    ~solution_information();
+
+    /** \brief Get the modifications the user made to the resolver
+     *  before this solution was produced.
+     */
+    const std::vector<resolver_interaction> *get_interactions() const
+    {
+      return interactions;
+    }
+
+    /** \brief Get an upper bound on the number of resolver steps
+     *  needed to produce this solution.
+     */
+    int get_ticks() const { return ticks; }
+
+    /** \brief Get the solution that was produced by the resolver. */
+    const generic_solution<aptitude_universe> *get_solution() const
+    {
+      return solution;
+    }
+
+    /** \return \b true if this solution is the "revert-all" solution,
+     *  the one that keeps all packages at their current version.
+    */
+    bool get_is_keep_all_solution() const
+    {
+      return is_keep_all_solution;
+    }
+  };
+
+  /** The solutions generated by this manager since the last change to
+   *  the cache, and the interaction that took place before each
+   *  solution was generated.
+   */
+  std::vector<const solution_information *> solutions;
 
   /** True if the solution search was aborted due to an uncaught
    *  exception.
@@ -222,6 +304,24 @@ private:
    */
   bool resolver_null;
 
+  /** If set to a non-empty string, an excerpt of the cache
+   *  corresponding to the packages touched by the resolver will be
+   *  written to this directory whenever the resolver finishes
+   *  running.
+   *
+   *  This is in the scope of background_control_mutex.
+   */
+  std::string resolver_trace_dir;
+
+  /** If set to a non-empty string, an excerpt of the cache
+   *  corresponding to the packages touched by the resolver will be
+   *  archived and written to this file whenever the resolver finishes
+   *  running.
+   *
+   *  This is in the scope of background_control_mutex.
+   */
+  std::string resolver_trace_file;
+
   /** The number of times the background thread has been suspended; it
    *  will only be allowed to run if this value is 0.
    */
@@ -236,13 +336,13 @@ private:
 
   /** A lock around pending_jobs, background_thread_killed,
    *  background_thread_suspend_count, background_thread_in_resolver,
-   *  and resolver_null.
+   *  resolver_null, and resolver_trace_dir.
    */
   cwidget::threads::mutex background_control_mutex;
 
   /** A condition signalled for pending_jobs,
-   *  background_thread_killed, background_thread_suspend_count, and
-   *  resolver_null.
+   *  background_thread_killed, background_thread_suspend_count,
+   *  resolver_null, and resolver_trace_dir.
    */
   cwidget::threads::condition background_control_cond;
 
@@ -275,12 +375,39 @@ private:
   class background_suspender;
   friend class background_suspender;
 
+  /** \brief Write a test control file, which can be used to replicate
+   * test output.
+   *
+   *  \param visited_packages   the packages visited by the problem resolver.
+   *
+   *  \param solution_number the setup and parameters used to compute
+   *  all solutions up to and including this solution index will be
+   *  dumped to the file.
+   */
+  void write_test_control_file(const std::string &outDir,
+			       const std::set<aptitude_resolver_package> &visited_packages,
+			       int solution_number);
+
+  /** \brief Dump the visited packages to a file if necessary.
+   *
+   *  Must be called with background_control_mutex held.
+   *
+   *  \param visited_packages  the packages touched by the
+   *                           problem resolver.
+   *  \param solution_number   the index of the last solution to
+   *                           dump a test for.
+   */
+  void dump_visited_packages(const std::set<aptitude_resolver_package> &visited_packages,
+			     int solution_number);
+
   /** Low-level code to get a solution; it does not take the global
    *  lock, does not stop a background thread, and may run either in
    *  the foreground or in the background.  It is called by
    *  background_thread_execution and get_solution.
    */
-  generic_solution<aptitude_universe> *do_get_solution(int max_steps, unsigned int solution_number);
+  const generic_solution<aptitude_universe> *
+  do_get_solution(int max_steps, unsigned int solution_number,
+		  std::set<aptitude_resolver_package> &visited_packages);
 
   /** The actual background thread. */
   void background_thread_execution();
@@ -321,13 +448,19 @@ private:
    */
   template<typename T>
   void resolver_manipulation(const T &t,
-			     void (generic_problem_resolver<aptitude_universe>::*action)(const T &, undo_group *));
+			     void (generic_problem_resolver<aptitude_universe>::*action)(const T &, undo_group *),
+			     const resolver_interaction &act);
 public:
   /** Create a new resolver manager for the given cache file. */
   resolver_manager(aptitudeDepCache *cache);
 
   virtual ~resolver_manager();
 
+  /** \brief Discard all past interactions and return the resolver to
+   *  its initial state.
+   */
+  void reset_resolver();
+
   /** If \b true, then a resolver has been created, indicating that
    *  problems may exist in the cache.
    */
@@ -342,6 +475,15 @@ public:
    */
   void set_debug(bool activate);
 
+  /** \brief Set the directory to which future resolver runs will
+   *  dump a cache containing only references to packages that
+   *  were touched by the resolver.
+   *
+   *  \param path the directory to which the subset should be
+   *  written, or an empty string to not write anything.
+   */
+  void set_resolver_trace_dir(const std::string &path);
+
   /** The number of solutions generated. */
   unsigned int generated_solution_count() const;
 
@@ -370,6 +512,11 @@ public:
   const generic_solution<aptitude_universe> &get_solution(unsigned int solution_num,
 							  int max_steps);
 
+  /** As get_solution, but returns whether the solution in question
+   *  is equal to the resolver's "keep-all" solution.
+   */
+  bool get_is_keep_all_solution(unsigned int solution_num, int max_steps);
+
   /** As get_solution, but run in a background thread if necessary.
    *
    *  \param solution_num the solution to retrieve
@@ -444,6 +591,15 @@ public:
 
 
 
+  /** \brief Reject all versions that will break holds or install
+   *  forbidden versions.
+   *
+   *  This is performed by default when the resolver is created in
+   *  aptitude_resolver::aptitude_resolver, but the same rejects can
+   *  be added explicitly using this routine.
+   */
+  void reject_break_holds();
+
   /** Requires that resolver_exists() is \b true.  Temporarily rejects
    *  any solutions generated by the currently active installer that
    *  involve installing the given version; the rejection will be
