@@ -1,6 +1,6 @@
 // problemresolver.h                  -*-c++-*-
 //
-//   Copyright (C) 2005, 2007 Daniel Burrows
+//   Copyright (C) 2005, 2007-2008 Daniel Burrows
 //
 //   This program is free software; you can redistribute it and/or
 //   modify it under the terms of the GNU General Public License as
@@ -32,9 +32,6 @@
 //  Boston, MA 02111-1307, USA.
 //
 //
-// Generic problem resolver (generic because I want to be able to do
-// regression-testing for once, if I can figure out how, and Packages
-// files make lousy regression-tests).
 
 #ifndef PROBLEMRESOLVER_H
 #define PROBLEMRESOLVER_H
@@ -58,6 +55,16 @@
 
 #include <generic/util/dense_setset.h>
 
+/** \brief Generic problem resolver
+ *
+ * 
+ *  Generic problem resolver (generic because I want to be able to do
+ *  regression-testing for once, if I can figure out how, and Packages
+ *  files make lousy regression-tests).
+ * 
+ *  \file problemresolver.h
+ */
+
 template<typename Obj1, typename Obj2, typename Uni>
 inline void eassert_fail_on_2objs_soln(const std::string &file,
 				       size_t line,
@@ -614,7 +621,7 @@ private:
   // Information regarding the weight given to various parameters;
   // packaged up in a struct so it can be easily used by the solution
   // constructors.
-  solution_weights weights;
+  solution_weights<PackageUniverse> weights;
 
   /** Solutions whose score is smaller than this value will be
    *  discarded rather than being enqueued.
@@ -1677,6 +1684,29 @@ private:
 
 	      return true;
 	    }
+
+	// Check whether a soft dependency is being left unresolved
+	// that this version could have satisfied and that's not
+	// approved-broken.
+	for(typename imm::set<dep>::const_iterator udi =
+	      s.get_unresolved_soft_deps().begin();
+	    udi != s.get_unresolved_soft_deps().end(); ++udi)
+	  {
+	    const dep ud(*udi);
+	    const bool is_approved_broken =
+	      user_approved_broken.find(ud) != user_approved_broken.end();
+	    if(!is_approved_broken && ud.solved_by(*ai))
+	      {
+		if(debug)
+		  {
+		    std::cout << ai->get_package().get_name() << " version " << ai->get_name() << " is avoided (by leaving " << ud << " unresolved) by the solution:" << std::endl;
+		    s.dump(std::cout);
+		    std::cout << std::endl;
+		  }
+
+		return true;
+	      }
+	  }
       }
 
     return false;
@@ -1922,9 +1952,11 @@ private:
   class real_generator
   {
     std::vector<solution> &target;
+    std::set<package> *visited_packages;
   public:
-    real_generator(std::vector<solution> &_target)
-      :target(_target)
+    real_generator(std::vector<solution> &_target,
+		   std::set<package> *_visited_packages)
+      :target(_target), visited_packages(_visited_packages)
     {
     }
 
@@ -1933,12 +1965,29 @@ private:
 			const a_iter &abegin, const a_iter &aend,
 			const u_iter &ubegin, const u_iter &uend,
 			const PackageUniverse &universe,
-			const solution_weights &weights) const
+			const solution_weights<PackageUniverse> &weights) const
     {
       target.push_back(solution::successor(s,
 					   abegin, aend,
 					   ubegin, uend,
 					   universe, weights));
+
+      // Touch all the packages that are involved in broken dependencies
+      if(visited_packages != NULL)
+	{
+	  const solution &generated(target.back());
+
+	  for(typename imm::set<dep>::const_iterator bi =
+		generated.get_broken().begin();
+	      bi != generated.get_broken().end(); ++bi)
+	    {
+	      for(typename dep::solver_iterator si =
+		    (*bi).solvers_begin(); !si.end(); ++si)
+		visited_packages->insert((*si).get_package());
+
+	      visited_packages->insert((*bi).get_source().get_package());
+	    }
+	}
     }
   };
 
@@ -1962,7 +2011,7 @@ private:
 			const a_iter &abegin, const a_iter &aend,
 			const u_iter &ubegin, const u_iter &uend,
 			const PackageUniverse &universe,
-			const solution_weights &weights) const
+			const solution_weights<PackageUniverse> &weights) const
     {
       ++count;
     }
@@ -1995,8 +2044,12 @@ private:
 				 const version &v,
 				 bool from_dep_source,
 				 imm::map<package, action> &conflict,
-				 const SolutionGenerator &generator) const
+				 const SolutionGenerator &generator,
+				 std::set<package> *visited_packages) const
   {
+    if(visited_packages != NULL)
+      visited_packages->insert(v.get_package());
+
     action conflictor;
 
     if(debug)
@@ -2072,39 +2125,65 @@ private:
    *  solvers, this will be a full conflict explaining the lack of
    *  solvers).
    *
+   *  \param visited_packages a set into which the packages used
+   *                          to compute the successors will be
+   *                          pushed.
+   *
    *  \param out a vector onto which the successors should be pushed.
    */
   template<typename SolutionGenerator>
   void generate_successors(const solution &s,
 			   const dep &d,
 			   imm::map<package, action> &conflict,
-			   const SolutionGenerator &generator) const
+			   const SolutionGenerator &generator,
+			   std::set<package> *visited_packages) const
   {
     version source = d.get_source();
 
+    if(visited_packages != NULL)
+      visited_packages->insert(source.get_package());
+
     typename imm::map<package, action>::node
       source_found = s.get_actions().lookup(source.get_package());
 
-    // Try moving the source, if it is legal to do so
-    if(source_found.isValid())
-      insert_conflictor(conflict, action(source, d, false, -1));
-    else
+    // Try moving the source, if it is legal to do so.
+    //
+    // The source is not moved for soft deps: these model Recommends,
+    // and experience has shown that users find it disturbing to have
+    // packages upgraded or removed due to their recommendations.
+    //
+    // Note that a conflictor is never generated for the source if the
+    // dep is soft, since there's no logical dependency in that case
+    // between the source being in the working solution and the
+    // dependency being unsatisfiable.
+    if(!d.is_soft())
       {
-	eassert_on_2objs_soln(source == source.get_package().current_version(),
-			      source,
-			      source.get_package().current_version(),
-			      s);
+	if(source_found.isValid())
+	  insert_conflictor(conflict, action(source, d, false, -1));
+	else
+	  {
+	    eassert_on_2objs_soln(source == source.get_package().current_version(),
+				  source,
+				  source.get_package().current_version(),
+				  s);
 
-	for(typename package::version_iterator vi = source.get_package().versions_begin();
-	    !vi.end(); ++vi)
-	  if(*vi != source)
-	    generate_single_successor(s, d, *vi, true, conflict, generator);
+	    for(typename package::version_iterator vi = source.get_package().versions_begin();
+		!vi.end(); ++vi)
+	      if(*vi != source)
+		generate_single_successor(s, d, *vi, true, conflict, generator,
+					  visited_packages);
+	  }
       }
 
     // Now try installing each target of the dependency.
     for(typename dep::solver_iterator si = d.solvers_begin();
 	!si.end(); ++si)
-      generate_single_successor(s, d, *si, false, conflict, generator);
+      {
+	if(visited_packages != NULL)
+	  visited_packages->insert((*si).get_package());
+	generate_single_successor(s, d, *si, false, conflict, generator,
+				  visited_packages);
+      }
 
     // Finally, maybe we can leave this dependency unresolved.
     if(d.is_soft())
@@ -2116,7 +2195,8 @@ private:
    *  (if any are available).  Note that for clarity, we now generate
    *  *all* successors before examining *any*.
    */
-  void process_solution(const solution &s)
+  void process_solution(const solution &s,
+			std::set<package> *visited_packages)
   {
     // Any forcings are immediately applied to 'curr', so that
     // forcings are performed ASAP.
@@ -2150,6 +2230,15 @@ private:
 	    bi!=starting_solution.get_broken().end(); ++bi)
 
 	  {
+	    if(visited_packages != NULL)
+	      {
+		for(typename dep::solver_iterator si = (*bi).solvers_begin();
+		    !si.end(); ++si)
+		  visited_packages->insert((*si).get_package());
+
+		visited_packages->insert((*bi).get_source().get_package());
+	      }
+
 	    // Check for the case where this dependency has been
 	    // fortuitously solved by forcing another broken
 	    // dependency.
@@ -2191,7 +2280,8 @@ private:
 
 	    int num_successors = 0;
 	    generate_successors(curr, *bi, conflict,
-				null_generator(num_successors));
+				null_generator(num_successors),
+				visited_packages);
 
 
 
@@ -2217,11 +2307,11 @@ private:
 		  std::cout << "Forced resolution of " << *bi << std::endl;
 
 		std::vector<solution> v;
-		real_generator g(v);
+		real_generator g(v, visited_packages);
 		// NB: this may do redundant work adding to 'conflict'.
 		// Use a generator object to avoid that?
 		generate_successors(curr, *bi, conflict,
-				    real_generator(v));
+				    g, visited_packages);
 
 		eassert_on_dep(v.size() == 1, s, *bi);
 
@@ -2263,7 +2353,9 @@ private:
 		      << std::endl;
 
 	  std::vector<solution> v;
-	  generate_successors(curr, *bi, conflict, real_generator(v));
+	  generate_successors(curr, *bi, conflict,
+			      real_generator(v, visited_packages),
+			      visited_packages);
 	  try_enqueue(v);
 	  nsols += v.size();
 	}
@@ -2280,7 +2372,9 @@ private:
 		    << std::endl;
 
 	std::vector<solution> v;
-	generate_successors(curr, *bi, conflict, real_generator(v));
+	generate_successors(curr, *bi, conflict,
+			    real_generator(v, visited_packages),
+			    visited_packages);
 	try_enqueue(v);
 	nsols += v.size();
       }
@@ -2330,7 +2424,7 @@ public:
   {
   }
 
-  const PackageUniverse &get_universe()
+  const PackageUniverse &get_universe() const
   {
     return universe;
   }
@@ -2419,6 +2513,19 @@ public:
     return weights.version_scores[ver.get_id()];
   }
 
+  /** \brief Add a score to all solutions that install the given
+   *  collection of versions.
+   *
+   *  Note that this does not mean "all solutions that result in the
+   *  given set of versions being installed".  The versions must be
+   *  newly installed in the solution; if any version is the current
+   *  version of its package, this call has no effect.
+   */
+  void add_joint_score(const imm::set<version> &versions, int score)
+  {
+    weights.add_joint_score(versions, score);
+  }
+
   /** Reject future solutions containing this version.
    */
   void reject_version(const version &ver, undo_group *undo = NULL)
@@ -2690,6 +2797,10 @@ public:
    *
    *  \param max_steps the maximum number of solutions to test.
    *
+   *  \param visited_packages
+   *           if not NULL, each package that influences the
+   *           resolver's choices will be placed here.
+   *
    *  \return a solution that fixes all broken dependencies
    *
    * \throws NoMoreSolutions if the potential solution list is exhausted.
@@ -2698,7 +2809,8 @@ public:
    *  \todo when throwing NoMoreSolutions or NoMoreTime, maybe we
    *        should include the "least broken" solution seen.
    */
-  solution find_next_solution(int max_steps)
+  solution find_next_solution(int max_steps,
+			      std::set<package> *visited_packages)
   {
     // This object is responsible for managing the instance variables
     // that control threaded operation: it sets solver_executing when
@@ -2707,7 +2819,8 @@ public:
     instance_tracker t(*this);
  
 
-    // Counter for debugging (see below)
+    // Counter for checking how long we've been running and for
+    // debugging (see below).
     int odometer = 0;
 
     if(deferred_dirty)
@@ -2733,7 +2846,7 @@ public:
 	{
 	  cwidget::threads::mutex::lock l(execution_mutex);
 	  if(solver_cancelled)
-	    throw InterruptedException();
+	    throw InterruptedException(odometer);
 	}
 
 	update_counts_cache();
@@ -2836,7 +2949,7 @@ public:
 	  }
 	// Nope, let's go enqueue successor nodes.
 	else
-	  process_solution(s);
+	  process_solution(s, visited_packages);
 
 	if(debug)
 	  std::cout << "Done generating successors." << std::endl;
@@ -2894,6 +3007,20 @@ public:
 	    out << " >" << std::endl;
 	  }
       }
+
+    for(typename std::vector<std::pair<imm::set<version>, int> >::const_iterator it =
+	  weights.get_joint_scores_list().begin();
+	it != weights.get_joint_scores_list().end(); ++it)
+      {
+	out << "  SCORE { ";
+	for(typename imm::set<version>::const_iterator vIt = it->first.begin();
+	    vIt != it->first.end(); ++vIt)
+	  {
+	    out << *vIt << " ";
+	  }
+	out << "} " << it->second << std::endl;
+      }
+
     out << "}" << std::endl;
   }
 };
