@@ -1,6 +1,6 @@
 // solution.h                                             -*-c++-*-
 //
-//   Copyright (C) 2005, 2007 Daniel Burrows
+//   Copyright (C) 2005, 2007-2008 Daniel Burrows
 //
 //   This program is free software; you can redistribute it and/or
 //   modify it under the terms of the GNU General Public License as
@@ -23,47 +23,17 @@
 #define SOLUTION_H
 
 #include <iostream>
+#include <map>
+
 #include <generic/util/immset.h>
 
-/** Represents the current score weights for a resolver.  Used to
- *  calculate scores at the time a solution is instantiated.
+/** \brief The solution class for the problem resolver.
+ * 
+ *  \file solution.h
  */
-struct solution_weights
-{
-  /** How much to reward long and/or broken solutions.  Typically
-   *  negative to penalize such things, or 0 to ignore them.
-   */
-  int step_score, broken_score, unfixed_soft_score;
 
-
-  /** How much to reward real solutions -- solutions that fix all
-   *  dependencies.  Make this big to immediately pick them up, or
-   *  small to ignore "bad" solutions (at the risk of running out of
-   *  time if no better solution pops up).
-   */
-  int full_solution_score;
-
-  /** The scores to apply to individual package versions.
-   */
-  int *version_scores;
-
-  solution_weights(int _step_score, int _broken_score,
-		   int _unfixed_soft_score, int _full_solution_score,
-		   unsigned long num_versions)
-    :step_score(_step_score), broken_score(_broken_score),
-     unfixed_soft_score(_unfixed_soft_score),
-     full_solution_score(_full_solution_score),
-     version_scores(new int[num_versions])
-  {
-    for(unsigned long i = 0; i < num_versions; ++i)
-      version_scores[i] = 0;
-  }
-
-  ~solution_weights()
-  {
-    delete[] version_scores;
-  }
-};
+template<typename PackageUniverse>
+class solution_weights;
 
 /** Represents a partial or complete solution to a dependency
  *  problem.  Solutions are transparently refcounted to save on
@@ -305,20 +275,7 @@ public:
   /** Generate the root node for a search in the given universe. */
   static generic_solution root_node(const imm::set<dep> &initial_broken,
 				    const PackageUniverse &universe,
-				    const solution_weights &weights)
-  {
-    int score = initial_broken.size() * weights.broken_score;
-
-    if(initial_broken.empty())
-      score += weights.full_solution_score;
-
-    return generic_solution(new solution_rep(imm::map<package, action>(),
-					     initial_broken,
-					     imm::set<dep>(),
-					     imm::map<version, dep>(),
-					     score,
-					     0));
-  }
+				    const solution_weights<PackageUniverse> &weights);
 
   /** Generate a successor to the given solution.
    *
@@ -332,132 +289,7 @@ public:
 				    const u_iter &ubegin,
 				    const u_iter &uend,
 				    const PackageUniverse &universe,
-				    const solution_weights &weights)
-  {
-    // NB: as I fully expect to move to a scheme of shared-memory
-    // sets/maps, the implicit copies here will go away eventually.
-    imm::set<dep> broken_deps = s.get_broken();
-    imm::map<package, action> actions = s.get_actions();
-    imm::map<version, dep> forbidden_versions = s.get_forbidden_versions();
-    imm::set<dep> unresolved_soft_deps = s.get_unresolved_soft_deps();
-    int action_score = s.get_action_score();
-
-    // Add notes about unresolved dependencies
-    for(u_iter ui = ubegin; ui != uend; ++ui)
-      {
-	const dep &d = *ui;
-
-	eassert(broken_deps.contains(d));
-	broken_deps.erase(d);
-	unresolved_soft_deps.insert(d);
-      }
-
-    for(a_iter ai = abegin; ai != aend; ++ai)
-      {
-	const action &a = *ai;
-	eassert(!actions.domain_contains(a.ver.get_package()));
-	eassert(a.ver != a.ver.get_package().current_version());
-
-	actions.put(a.ver.get_package(), a);
-
-	action_score += weights.step_score;
-	action_score += weights.version_scores[a.ver.get_id()];
-	action_score -= weights.version_scores[a.ver.get_package().current_version().get_id()];
-
-	if(a.from_dep_source)
-	  {
-	    for(typename dep::solver_iterator si = a.d.solvers_begin();
-		!si.end(); ++si)
-	      forbidden_versions.put(*si, a.d);
-	  }
-
-
-
-	// Update the set of broken dependencies, trying to re-use as
-	// much of the former set as possible.
-	version old_version=s.version_of(a.ver.get_package());
-	solution_map_wrapper tmpsol(actions);
-
-	// Check reverse deps of the old version
-	for(typename version::revdep_iterator rdi=old_version.revdeps_begin();
-	    !rdi.end(); ++rdi)
-	  {
-	    dep rd = *rdi;
-
-	    if((rd).broken_under(tmpsol))
-	      {
-		if(!unresolved_soft_deps.contains(rd))
-		  broken_deps.insert(rd);
-	      }
-	    else
-	      {
-		broken_deps.erase(rd);
-		unresolved_soft_deps.erase(rd);
-	      }
-	  }
-
-	// Check reverse deps of the new version
-	//
-	// Because reverse deps of a version might be fixed by its
-	// removal, we need to check brokenness and insert or erase as
-	// appropriate.
-	for(typename version::revdep_iterator rdi = a.ver.revdeps_begin();
-	    !rdi.end(); ++rdi)
-	  {
-	    dep rd = *rdi;
-
-	    if((rd).broken_under(tmpsol))
-	      {
-		if(!unresolved_soft_deps.contains(rd))
-		  broken_deps.insert(rd);
-	      }
-	    else
-	      {
-		broken_deps.erase(rd);
-		unresolved_soft_deps.erase(rd);
-	      }
-	  }
-
-	// Remove all forward deps of the old version (they're
-	// automagically fixed)
-	for(typename version::dep_iterator di=old_version.deps_begin();
-	    !di.end(); ++di)
-	  {
-	    dep d = *di;
-
-	    broken_deps.erase(d);
-	    unresolved_soft_deps.erase(d);
-	  }
-
-	// Check forward deps of the new version (no need to erase
-	// non-broken dependencies since they're automatically
-	// non-broken at the start)
-	for(typename version::dep_iterator di=a.ver.deps_begin();
-	    !di.end(); ++di)
-	  {
-	    dep d = *di;
-
-	    if(!unresolved_soft_deps.contains(d) &&
-	       (d).broken_under(tmpsol))
-	      broken_deps.insert(d);
-	  }
-      }
-
-
-    int score
-      = action_score + broken_deps.size()*weights.broken_score
-      + unresolved_soft_deps.size()*weights.unfixed_soft_score;
-
-    if(broken_deps.empty())
-      score += weights.full_solution_score;
-
-    return generic_solution(new solution_rep(actions,
-					     broken_deps,
-					     unresolved_soft_deps,
-					     forbidden_versions,
-					     score,
-					     action_score));
-  }
+				    const solution_weights<PackageUniverse> &weights);
 
   ~generic_solution()
   {
@@ -745,4 +577,326 @@ public:
   };
 }; // End solution wrapper
 
+
+/** Represents the current score weights for a resolver.  Used to
+ *  calculate scores at the time a solution is instantiated.
+ */
+template<typename  PackageUniverse>
+struct solution_weights
+{
+  typedef typename PackageUniverse::package package;
+  typedef typename PackageUniverse::version version;
+  typedef typename PackageUniverse::dep dep;
+  typedef typename generic_solution<PackageUniverse>::action action;
+
+  /** \brief Represents a score assigned to a collection of actions. */
+  class joint_score
+  {
+    imm::map<package, action> actions;
+    int score;
+
+  public:
+    joint_score(const imm::map<package, action> &_actions, int _score)
+      : actions(_actions), score(_score)
+    {
+    }
+
+    const imm::map<package, action> &get_actions() const { return actions; }
+    int get_score() const { return score; }
+  };
+
+  typedef std::map<action, std::vector<joint_score> > joint_score_set;
+
+  /** How much to reward long and/or broken solutions.  Typically
+   *  negative to penalize such things, or 0 to ignore them.
+   */
+  int step_score, broken_score, unfixed_soft_score;
+
+
+  /** How much to reward real solutions -- solutions that fix all
+   *  dependencies.  Make this big to immediately pick them up, or
+   *  small to ignore "bad" solutions (at the risk of running out of
+   *  time if no better solution pops up).
+   */
+  int full_solution_score;
+
+  /** The scores to apply to individual package versions.
+   */
+  int *version_scores;
+
+private:
+  /** \brief Scores that apply to simultaneous collections of
+   *  versions.
+   */
+  joint_score_set joint_scores;
+
+  /** \brief A list of the joint scores added to this
+   *  set of weights, in order.
+   *
+   *  Each entry is a pair of the versions that are affected to and
+   *  the score to add.
+   */
+  std::vector<std::pair<imm::set<version>, int> > joint_scores_list;
+
+public:
+  solution_weights(int _step_score, int _broken_score,
+		   int _unfixed_soft_score, int _full_solution_score,
+		   unsigned long num_versions)
+    :step_score(_step_score), broken_score(_broken_score),
+     unfixed_soft_score(_unfixed_soft_score),
+     full_solution_score(_full_solution_score),
+     version_scores(new int[num_versions])
+  {
+    for(unsigned long i = 0; i < num_versions; ++i)
+      version_scores[i] = 0;
+  }
+
+  ~solution_weights()
+  {
+    delete[] version_scores;
+  }
+
+private:
+  class build_joint_score_action_set
+  {
+    imm::map<package, action> &output;
+    bool &any_is_current;
+  public:
+    build_joint_score_action_set(imm::map<package, action> &_output,
+				 bool &_any_is_current)
+      : output(_output), any_is_current(_any_is_current)
+    {
+    }
+
+    void operator()(const version &version) const
+    {
+      if(version == version.get_package().current_version())
+	any_is_current = true;
+
+      output.put(version.get_package(),
+		 action(version, dep(), false, 0));
+    }
+  };
+
+  class add_to_joint_scores
+  {
+    typedef typename solution_weights<PackageUniverse>::joint_score joint_score;
+    typedef typename solution_weights<PackageUniverse>::joint_score_set joint_score_set;
+    typedef typename generic_solution<PackageUniverse>::action action;
+
+    joint_score_set &s;
+    joint_score score;
+  public:
+    add_to_joint_scores(joint_score_set &_s, const joint_score &_score)
+      : s(_s), score(_score)
+    {
+    }
+
+    void operator()(const std::pair<package, action> &entry) const
+    {
+      const typename joint_score_set::iterator found =
+	s.find(entry.second);
+
+      if(found == s.end())
+	s[entry.second].push_back(score);
+      else
+	found->second.push_back(score);
+    }
+  };
+
+public:
+  void add_joint_score(const imm::set<version> &versions, int score)
+  {
+    // Build a map internally: it's easier to compare a map to another
+    // map than to compare it to a set.  (could be fixed by allowing
+    // disjoint sets to be compared under inclusion with an
+    // appropriate cross-compare)
+    imm::map<package, action> actions_map;
+    bool any_is_current = false;
+    versions.for_each(build_joint_score_action_set(actions_map,
+						   any_is_current));
+
+    if(any_is_current)
+      return;
+
+    joint_scores_list.push_back(std::make_pair(versions, score));
+
+    actions_map.for_each(add_to_joint_scores(joint_scores,
+					     typename solution_weights<PackageUniverse>::joint_score(actions_map, score)));
+  }
+
+  const joint_score_set &get_joint_scores() const { return joint_scores; }
+  const std::vector<std::pair<imm::set<version>, int> > &
+  get_joint_scores_list() const { return joint_scores_list; }
+};
+
+template<typename PackageUniverse>
+inline generic_solution<PackageUniverse>
+generic_solution<PackageUniverse>::root_node(const imm::set<dep> &initial_broken,
+					     const PackageUniverse &universe,
+					     const solution_weights<PackageUniverse> &weights)
+{
+  int score = initial_broken.size() * weights.broken_score;
+
+  if(initial_broken.empty())
+    score += weights.full_solution_score;
+
+  return generic_solution(new solution_rep(imm::map<package, action>(),
+					   initial_broken,
+					   imm::set<dep>(),
+					   imm::map<version, dep>(),
+					   score,
+					   0));
+}
+
+
+template<typename PackageUniverse>
+template<typename a_iter, typename u_iter>
+inline generic_solution<PackageUniverse>
+generic_solution<PackageUniverse>::successor(const generic_solution &s,
+					     const a_iter &abegin,
+					     const a_iter &aend,
+					     const u_iter &ubegin,
+					     const u_iter &uend,
+					     const PackageUniverse &universe,
+					     const solution_weights<PackageUniverse> &weights)
+{
+  imm::set<dep> broken_deps = s.get_broken();
+  imm::map<package, action> actions = s.get_actions();
+  imm::map<version, dep> forbidden_versions = s.get_forbidden_versions();
+  imm::set<dep> unresolved_soft_deps = s.get_unresolved_soft_deps();
+  int action_score = s.get_action_score();
+
+  // Add notes about unresolved dependencies
+  for(u_iter ui = ubegin; ui != uend; ++ui)
+    {
+      const dep &d = *ui;
+
+      eassert(broken_deps.contains(d));
+      broken_deps.erase(d);
+      unresolved_soft_deps.insert(d);
+    }
+
+  for(a_iter ai = abegin; ai != aend; ++ai)
+    {
+      const action &a = *ai;
+      eassert(!actions.domain_contains(a.ver.get_package()));
+      eassert(a.ver != a.ver.get_package().current_version());
+
+      actions.put(a.ver.get_package(), a);
+
+      action_score += weights.step_score;
+      action_score += weights.version_scores[a.ver.get_id()];
+      action_score -= weights.version_scores[a.ver.get_package().current_version().get_id()];
+
+      // Look for joint score constraints triggered by adding this
+      // action.
+      const typename solution_weights<PackageUniverse>::joint_score_set::const_iterator
+	joint_scores_found = weights.get_joint_scores().find(a);
+      if(joint_scores_found != weights.get_joint_scores().end())
+	{
+	  typedef typename solution_weights<PackageUniverse>::joint_score joint_score;
+	  for(typename std::vector<joint_score>::const_iterator it =
+		joint_scores_found->second.begin();
+	      it != joint_scores_found->second.end(); ++it)
+	    {
+	      if(actions.is_supermap_of(it->get_actions()))
+		action_score += it->get_score();
+	    }
+	}
+
+      if(a.from_dep_source)
+	{
+	  for(typename dep::solver_iterator si = a.d.solvers_begin();
+	      !si.end(); ++si)
+	    forbidden_versions.put(*si, a.d);
+	}
+
+
+
+      // Update the set of broken dependencies, trying to re-use as
+      // much of the former set as possible.
+      version old_version=s.version_of(a.ver.get_package());
+      solution_map_wrapper tmpsol(actions);
+
+      // Check reverse deps of the old version
+      for(typename version::revdep_iterator rdi=old_version.revdeps_begin();
+	  !rdi.end(); ++rdi)
+	{
+	  dep rd = *rdi;
+
+	  if((rd).broken_under(tmpsol))
+	    {
+	      if(!unresolved_soft_deps.contains(rd))
+		broken_deps.insert(rd);
+	    }
+	  else
+	    {
+	      broken_deps.erase(rd);
+	      unresolved_soft_deps.erase(rd);
+	    }
+	}
+
+      // Check reverse deps of the new version
+      //
+      // Because reverse deps of a version might be fixed by its
+      // removal, we need to check brokenness and insert or erase as
+      // appropriate.
+      for(typename version::revdep_iterator rdi = a.ver.revdeps_begin();
+	  !rdi.end(); ++rdi)
+	{
+	  dep rd = *rdi;
+
+	  if((rd).broken_under(tmpsol))
+	    {
+	      if(!unresolved_soft_deps.contains(rd))
+		broken_deps.insert(rd);
+	    }
+	  else
+	    {
+	      broken_deps.erase(rd);
+	      unresolved_soft_deps.erase(rd);
+	    }
+	}
+
+      // Remove all forward deps of the old version (they're
+      // automagically fixed)
+      for(typename version::dep_iterator di=old_version.deps_begin();
+	  !di.end(); ++di)
+	{
+	  dep d = *di;
+
+	  broken_deps.erase(d);
+	  unresolved_soft_deps.erase(d);
+	}
+
+      // Check forward deps of the new version (no need to erase
+      // non-broken dependencies since they're automatically
+      // non-broken at the start)
+      for(typename version::dep_iterator di=a.ver.deps_begin();
+	  !di.end(); ++di)
+	{
+	  dep d = *di;
+
+	  if(!unresolved_soft_deps.contains(d) &&
+	     (d).broken_under(tmpsol))
+	    broken_deps.insert(d);
+	}
+    }
+
+  int score
+    = action_score + broken_deps.size()*weights.broken_score
+    + unresolved_soft_deps.size()*weights.unfixed_soft_score;
+
+  if(broken_deps.empty())
+    score += weights.full_solution_score;
+
+  return generic_solution(new solution_rep(actions,
+					   broken_deps,
+					   unresolved_soft_deps,
+					   forbidden_versions,
+					   score,
+					   action_score));
+}
+
 #endif // SOLUTION_H
