@@ -1,6 +1,6 @@
 // immset.h                                     -*-c++-*-
 //
-//   Copyright (C) 2005-2006 Daniel Burrows
+//   Copyright (C) 2005-2006, 2008 Daniel Burrows
 //
 //   This program is free software; you can redistribute it and/or
 //   modify it under the terms of the GNU General Public License as
@@ -17,18 +17,6 @@
 //   the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 //   Boston, MA 02111-1307, USA.
 //
-// This file defines a class to represent immutable sets.  These sets
-// behave like std::set, except that their contents cannot be changed;
-// they have no erase() or insert() operators, and only
-// const_iterators [0].  This restriction allows immutable sets to be
-// implemented in a way that makes both copying and creating a new set
-// by adding an element very efficient (O(1) and O(lg n) respectively;
-// with std::set these are both O(n)).
-//
-// [0] this refers to the wtree_node class; imm::set is a wrapper
-// around a wtree_node pointer that emulates a mutative interface by
-// creating a new tree and modifying its encapsulated pointer to point
-// there instead.
 
 #ifndef IMMSET_H
 #define IMMSET_H
@@ -38,6 +26,25 @@
 
 #include <vector>
 
+/** \brief A class to represent immutable sets
+ *
+ * 
+ *  This file defines a class to represent immutable sets.  These sets
+ *  behave like std::set, except that their contents cannot be changed;
+ *  they have no erase() or insert() operators, and only
+ *  const_iterators [0].  This restriction allows immutable sets to be
+ *  implemented in a way that makes both copying and creating a new set
+ *  by adding an element very efficient (O(1) and O(lg n) respectively;
+ *  with std::set these are both O(n)).
+ *  
+ *  [0] this refers to the wtree_node class; imm::set is a wrapper
+ *  around a wtree_node pointer that emulates a mutative interface by
+ *  creating a new tree and modifying its encapsulated pointer to point
+ *  there instead.
+ * 
+ *  \file immset.h
+ */
+
 namespace imm
 {
   /** A generic node in a weighted tree as described in "Implementing
@@ -1013,6 +1020,21 @@ namespace imm
       contents.dump(out);
     }
 
+    /** \brief Return \b true if each binding is related under
+     *  compare to a binding in other of an equivalent key.
+     */
+    template<typename BindingCmp>
+    bool is_supermap_of_under(const map &other,
+			      const BindingCmp &compare = BindingCmp()) const
+    {
+      return contents.contains(other.contents, compare);
+    }
+
+    bool is_supermap_of(const map &other)
+    {
+      return contents.contains(other.contents, std::equal_to<std::pair<Key, Val> >());
+    }
+
     bool domain_intersects(const map &other) const
     {
       return contents.intersects(other.contents);
