@@ -6,7 +6,7 @@
 
 #include "undo.h"
 
-#include <eassert.h>
+#include <cwidget/generic/util/eassert.h>
 
 using namespace std;
 
