@@ -37,14 +37,13 @@
 
 #include <generic/apt/apt.h>
 
-#include <cwidget/generic/util/strhash.h>
-
 #include <apt-pkg/error.h>
 
 #include <map>
 
 using namespace std;
 namespace cw = cwidget;
+namespace match = aptitude::matching;
 
 class GroupParseException
 {
@@ -353,7 +352,9 @@ class section_policy_parser : public string_policy_parser
 public:
   group_policy_parse_node *create_node(const vector<string> &args)
   {
-    int split_mode=pkg_grouppolicy_section_factory::split_none;
+    typedef pkg_grouppolicy_section_factory::split_mode_type split_mode_type;
+
+    split_mode_type split_mode = pkg_grouppolicy_section_factory::split_none;
     bool passthrough=false;
 
     if(args.size() >= 1)
@@ -364,8 +365,10 @@ public:
 	  split_mode=pkg_grouppolicy_section_factory::split_topdir;
 	else if(!strcasecmp(args[0].c_str(), "subdir"))
 	  split_mode=pkg_grouppolicy_section_factory::split_subdir;
+	else if(!strcasecmp(args[0].c_str(), "subdirs"))
+	  split_mode=pkg_grouppolicy_section_factory::split_subdirs;
 	else
-	  throw GroupParseException(_("Bad section name '%s' (use 'none', 'topdir', or 'subdir')"), args[0].c_str());
+	  throw GroupParseException(_("Bad section name '%s' (use 'none', 'topdir', 'subdir', or 'subdirs')"), args[0].c_str());
       }
 
     if(args.size() >= 2)
@@ -383,7 +386,7 @@ public:
       throw GroupParseException(_("Too many arguments to by-section grouping policy"));
 
     return new policy_node2<pkg_grouppolicy_section_factory,
-      int, bool>(split_mode, passthrough);
+      split_mode_type, bool>(split_mode, passthrough);
   }
 };
 
@@ -431,11 +434,11 @@ public:
 
     if(is_missing)
       {
-	pkg_matcher *m = parse_pattern("~T");
+	match::pkg_matcher *m = match::parse_pattern("~T");
 	begin = begin2;
 	++begin;
 
-	return new policy_node1<pkg_grouppolicy_filter_factory, pkg_matcher *>(m);
+	return new policy_node1<pkg_grouppolicy_filter_factory, match::pkg_matcher *>(m);
       }
     else
       {
@@ -443,8 +446,8 @@ public:
 	terminators.push_back(",");
 	terminators.push_back(")");
 
-	auto_ptr<pkg_matcher> m(parse_pattern(begin, end, terminators,
-					      false, true, false));
+	auto_ptr<match::pkg_matcher> m(match::parse_pattern(begin, end, terminators,
+							    false, true, false));
 
 	if(m.get() == NULL)
 	  throw GroupParseException(_("Unable to parse pattern at '%s'"),
@@ -459,7 +462,7 @@ public:
 	    if(begin != end)
 	      ++begin;
 
-	    return new policy_node1<pkg_grouppolicy_filter_factory, pkg_matcher *>(m.release());
+	    return new policy_node1<pkg_grouppolicy_filter_factory, match::pkg_matcher *>(m.release());
 	  }
       }
   }
@@ -600,9 +603,9 @@ class pattern_policy_parser : public group_policy_parser
 
 	    const string::const_iterator begin0 = begin;
 
-	    auto_ptr<pkg_matcher> matcher(parse_pattern(begin, end,
-							terminators,
-							false, true, false));
+	    auto_ptr<match::pkg_matcher> matcher(match::parse_pattern(begin, end,
+								      terminators,
+								      false, true, false));
 
 	    bool passthrough = false;
 
