@@ -1,6 +1,6 @@
 // pkg_grouppolicy.h       -*-c++-*-
 //
-//  Copyright 1999-2002, 2005, 2007 Daniel Burrows
+//  Copyright 1999-2002, 2005, 2007-2008 Daniel Burrows
 //
 //  This program is free software; you can redistribute it and/or modify
 //  it under the terms of the GNU General Public License as published by
@@ -17,32 +17,6 @@
 //  the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 //  Boston, MA 02111-1307, USA.
 //
-//  Ok, this requires (a little) explanation -- basically, the idea is that
-// we need a flexible way to group (and filter) packages.  These classes
-// provide that.  A grouping policy knows what tree it should add things to,
-// and can add subtrees or packages at its own discretion.  A policy
-// *factory* is used as a way to chain policies together -- rather than
-// directly adding packages to trees they create, most packages will
-// instantiate (using a factory) a policy for each subtree.
-//
-//  In other words: say we want to group by top-level section, then by
-// subsection.  We create a pkg_groupsectionpolicy_factory with the
-// proper arguments to group by top-level section, with its 'chain' argument
-// set to another pkg_groupsectionpolicy_factory (which, however, groups
-// by subsection) with its chain argument set to a pkg_grouppolicy_end_factory.
-//
-//  Note that the policy classes are not themselves visible to the user,
-// simply to avoid unnecessary recompiles.  The factories are the only way
-// to get a policy class.
-//
-//  The factories could probably be functions/function pointers as well,
-// but classes are more flexible in this instance.
-//
-//  Because of the way factories and policies generally work you should never
-// destroy a factory before destroying the new objects created by it.
-
-// An abstract base class for objects containing the logic to sort packages
-// into groups
 
 #ifndef PKG_GROUPPOLICY_H
 #define PKG_GROUPPOLICY_H
@@ -52,6 +26,39 @@
 
 #include <vector>
 
+/** \brief Provides a flexible way to group and filter packages
+ *
+ * 
+ *  Ok, this requires (a little) explanation -- basically, the idea is that
+ *  we need a flexible way to group (and filter) packages.  These classes
+ *  provide that.  A grouping policy knows what tree it should add things to,
+ *  and can add subtrees or packages at its own discretion.  A policy
+ *  *factory* is used as a way to chain policies together -- rather than
+ *  directly adding packages to trees they create, most packages will
+ *  instantiate (using a factory) a policy for each subtree.
+ *  
+ *  In other words: say we want to group by top-level section, then by
+ *  subsection.  We create a pkg_groupsectionpolicy_factory with the
+ *  proper arguments to group by top-level section, with its 'chain' argument
+ *  set to another pkg_groupsectionpolicy_factory (which, however, groups
+ *  by subsection) with its chain argument set to a pkg_grouppolicy_end_factory.
+ *  
+ *  Note that the policy classes are not themselves visible to the user,
+ *  simply to avoid unnecessary recompiles.  The factories are the only way
+ *  to get a policy class.
+ *  
+ *  The factories could probably be functions/function pointers as well,
+ *  but classes are more flexible in this instance.
+ *  
+ *  Because of the way factories and policies generally work you should never
+ *  destroy a factory before destroying the new objects created by it.
+ *  
+ *  An abstract base class for objects containing the logic to sort packages
+ *  into groups
+ * 
+ *  \file pkg_grouppolicy.h
+ */
+
 // Used to set the visible status if a package is available.
 typedef sigc::signal2<void,
 		      const pkgCache::PkgIterator &,
@@ -61,7 +68,13 @@ typedef sigc::signal2<void,
 typedef sigc::signal1<void, std::wstring> desc_signal;
 
 class pkg_subtree;
-class pkg_matcher;
+namespace aptitude
+{
+  namespace matching
+  {
+    class pkg_matcher;
+  }
+}
 
 class pkg_grouppolicy
 {
@@ -107,24 +120,50 @@ class pkg_grouppolicy_end_factory:public pkg_grouppolicy_factory
 // The following policy groups packages by section.
 class pkg_grouppolicy_section_factory:public pkg_grouppolicy_factory
 {
+public:
+  /** \brief Which part of the section to use: */
+  enum split_mode_type
+    {
+      /** \brief Use the whole section, uninterpreted.
+       *
+       *  This gives you names like "games" and "non-free/editors".
+       */
+      split_none = 0,
+
+      /** \brief Use the part of the section before the first slash,
+       *  or "main" if there is no slash ("/").
+       *
+       *  This gives you names like "main", "non-free", etc.
+       */
+      split_topdir = 1,
+
+      /** \brief Use the part of the section after the first slash,
+       *  or the whole section if there is no slash.
+       */
+      split_subdir = 2,
+
+      /** \brief Remove any text preceding a slash, then split the
+       *  remainder at slashes and build a multi-level hierarchy of
+       *  sections.
+       *
+       *  If you have packages in the sections "games/arcade" and
+       *  "games/strategy", this will produce a tree named "games",
+       *  with trees named "arcade" and "strategy" beneath it.
+       */
+      split_subdirs = 3
+    };
+
+private:
   pkg_grouppolicy_factory *chain;
 
-  int split_mode;
+  split_mode_type split_mode;
 
   // If this is true, virtual packages, packages in unknown sections, and
   // task packages will be 'passed through' to the next policy without having
   // a new level of tree structure created.
   bool passthrough;
 public:
-  // How to split the 'section' value:
-  static const int split_none=0;
-  // Don't.  This gives you names like "games" and "non-free/editors".
-  static const int split_topdir=1;
-  // Split it and keep the top-level half (adding an implied "main" to
-  // packages without a first half).  So you get "main", "non-free", etc.
-  static const int split_subdir=2;
-
-  pkg_grouppolicy_section_factory(int _split_mode,
+  pkg_grouppolicy_section_factory(split_mode_type _split_mode,
 				  bool _passthrough,
 				  pkg_grouppolicy_factory *_chain)
     :chain(_chain), split_mode(_split_mode), passthrough(_passthrough) {}
@@ -155,9 +194,9 @@ class pkg_grouppolicy_filter_factory:public pkg_grouppolicy_factory
 {
   pkg_grouppolicy_factory *chain;
 
-  pkg_matcher *filter;
+  aptitude::matching::pkg_matcher *filter;
 public:
-  pkg_grouppolicy_filter_factory(pkg_matcher *_filter,
+  pkg_grouppolicy_filter_factory(aptitude::matching::pkg_matcher *_filter,
 				 pkg_grouppolicy_factory *_chain)
     :chain(_chain), filter(_filter) {}
 
@@ -266,11 +305,11 @@ class pkg_grouppolicy_matchers_factory:public pkg_grouppolicy_factory
 public:
   struct match_pair
   {
-    pkg_matcher *matcher;
+    aptitude::matching::pkg_matcher *matcher;
     std::wstring tree_name;
     bool passthrough;
 
-    match_pair(pkg_matcher *_matcher,
+    match_pair(aptitude::matching::pkg_matcher *_matcher,
 	       const std::wstring &_tree_name,
 	       bool _passthrough)
       :matcher(_matcher), tree_name(_tree_name), passthrough(_passthrough)
