@@ -1,6 +1,6 @@
 // pkg_view.cc
 //
-//  Copyright 2000-2005, 2007 Daniel Burrows
+//  Copyright 2000-2005, 2007-2008 Daniel Burrows
 //
 //   This program is free software; you can redistribute it and/or
 //   modify it under the terms of the GNU General Public License as
@@ -65,6 +65,8 @@ namespace cwidget
   using namespace widgets;
 }
 
+namespace match = aptitude::matching;
+
 class pkg_handling_label:public cw::label
 {
   cw::config::column_definition_list *columns;
@@ -187,12 +189,14 @@ public:
   {
     if(pkg.end())
       {
-	set_fragment(cw::text_fragment(""));
+	set_no_package();
 	return;
       }
 
-    std::vector<pkg_matcher *> search_leaves;
-    search_leaves.push_back(parse_pattern("~i!~M"));
+    std::vector<match::pkg_matcher *> search_leaves;
+    // Search for package (versions) that are not automatically
+    // installed and that are or will be installed.
+    search_leaves.push_back(match::parse_pattern("?not(?automatic)?any-version(?or(?version(CANDIDATE)?action(install), ?version(CURRENT)?installed))"));
     try
       {
 	bool success = false;
@@ -204,15 +208,20 @@ public:
       }
     catch(...)
       {
-	for(std::vector<pkg_matcher *>::const_iterator it
+	for(std::vector<match::pkg_matcher *>::const_iterator it
 	      = search_leaves.begin(); it != search_leaves.end(); ++it)
 	  delete *it;
       }
 
-    for(std::vector<pkg_matcher *>::const_iterator it
+    for(std::vector<match::pkg_matcher *>::const_iterator it
 	  = search_leaves.begin(); it != search_leaves.end(); ++it)
       delete *it;
   }
+
+  void set_no_package()
+  {
+    set_fragment(wrapbox(cw::text_fragment(_("If you select a package, an explanation of why it should be installed or removed will appear in this space."))));
+  }
 };
 typedef cw::util::ref_ptr<pkg_why_widget> pkg_why_widget_ref;
 
@@ -301,6 +310,12 @@ class info_area_multiplex:public cw::multiplex
   cw::text_layout_ref reasons;
   cw::table_ref reasons_table;
 
+  /** True if the package members are valid.
+   *
+   *  This needs to be here because it's not always safe to check end()
+   *  if the cache has been reloaded.
+   */
+  bool have_pkg;
   pkgCache::PkgIterator lastPkg;
   pkgCache::VerIterator lastVer;
   wstring lastDesc;
@@ -312,6 +327,11 @@ class info_area_multiplex:public cw::multiplex
    */
   cw::widget_ref autoswitch;
 
+  void clear_package()
+  {
+    have_pkg = false;
+  }
+
 protected:
   info_area_multiplex(const hier_editor_ref &_editor,
 		      const pkg_description_widget_ref &_description,
@@ -325,8 +345,10 @@ protected:
      description(_description), description_table(_description_table),
      why(_why), why_table(_why_table),
      reasons(_reasons), reasons_table(_reasons_table),
-     hadBreakage(false), autoswitch(NULL)
+     have_pkg(false), hadBreakage(false), autoswitch(NULL)
   {
+    cache_closed.connect(sigc::mem_fun(*this,
+				       &info_area_multiplex::clear_package));
     package_states_changed.connect(sigc::mem_fun(*this,
 						 &info_area_multiplex::reset_package));
   }
@@ -392,6 +414,7 @@ public:
     // moved to a broken package.
     if(hasBreakage &&
        (!hadBreakage ||
+	!have_pkg ||
 	!(pkg==lastPkg && ver==lastVer)) &&
        aptcfg->FindB(PACKAGE "::UI::Auto-Show-Reasons", true))
       {
@@ -404,7 +427,7 @@ public:
 
     // We always set the package anyway in case something changed,
     // but only scroll to the top in this case:
-    if(pkg!=lastPkg || ver!=lastVer)
+    if(!have_pkg || pkg != lastPkg || ver != lastVer)
       {
 	lastPkg=pkg;
 	lastVer=ver;
@@ -420,6 +443,12 @@ public:
       }
 
     hadBreakage=hasBreakage;
+
+    // Important: we have to set have_pkg according to whether
+    // pkg.end() is true, because if it is, then we might be closing
+    // the cache, and then we might not recognize that the package is
+    // null when we open the cache again.
+    have_pkg = !pkg.end();
   }
 
   /** Cycles the multiplex, taking autoswitch behavior into account. */
@@ -444,7 +473,8 @@ public:
    */
   void reset_package()
   {
-    set_package(lastPkg, lastVer);
+    if(have_pkg)
+      set_package(lastPkg, lastVer);
   }
 
   /** Set the description directly, without reference to a package.
@@ -462,9 +492,10 @@ public:
 
 	description->set_fragment(make_desc_fragment(s));
 	reasons->set_fragment(cw::sequence_fragment(make_desc_fragment(s),
-						cw::newline_fragment(),
-						nopackage(),
-						NULL));
+						    cw::newline_fragment(),
+						    nopackage(),
+						    NULL));
+	why->set_no_package();
 
 	description->move_to_top();
 	reasons->move_to_top();
@@ -474,6 +505,53 @@ public:
 
 typedef cw::util::ref_ptr<info_area_multiplex> info_area_multiplex_ref;
 
+namespace
+{
+  bool view_is_active(cw::widget_ref view)
+  {
+    cw::util::ref_ptr<cw::container> owner = view->get_owner();
+
+    while(owner.valid())
+      {
+	if(owner->get_active_widget() != view)
+	  return false;
+
+	view = owner;
+	owner = view->get_owner();
+      }
+
+    return true;
+  }
+
+  bool do_info_multiplex_cycle_information_active(cw::widget &valveBare)
+  {
+    cw::widget_ref valve(&valveBare);
+    return view_is_active(valve);
+  }
+
+  bool do_info_multiplex_cycle_information(cw::widget &valveBare,
+					   info_area_multiplex &multiplexBare)
+  {
+    cw::widget_ref valve(&valveBare);
+    info_area_multiplex_ref multiplex(&multiplexBare);
+
+    if(!view_is_active(valve))
+      return false;
+    else
+      {
+	multiplex->cycle();
+	return true;
+      }
+  }
+
+  void do_update_info_area_show_tabs(cw::widgets::multiplex &mBare)
+  {
+    cw::widgets::multiplex_ref m(&mBare);
+
+    m->set_show_tabs(aptcfg->FindB(PACKAGE "::UI::InfoAreaTabs", false));
+  }
+}
+
 cw::widget_ref make_package_view(list<package_view_item> &format,
 				const cw::widget_ref &mainwidget,
 				menu_redirect *menu_handler,
@@ -583,16 +661,30 @@ cw::widget_ref make_package_view(list<package_view_item> &format,
 				    sigc::bind(sigc::mem_fun(*rval.unsafe_get_ref(), &cw::table::focus_widget_bare),
 					       m.weak_ref()));
 
+	    package_cycle_information_enabled.connect(sigc::bind(sigc::ptr_fun(do_info_multiplex_cycle_information_active), rval.weak_ref()));
+	    package_cycle_information.connect(sigc::bind(sigc::ptr_fun(do_info_multiplex_cycle_information), rval.weak_ref(), m.weak_ref()));
+
 	    e->hidden_sig.connect(sigc::bind(sigc::mem_fun(*rval.unsafe_get_ref(), &cw::table::focus_widget_bare),
 					     mainwidget.weak_ref()));
 
 	    if(thetree.valid())
 	      e->commit_changes.connect(sigc::mem_fun(*thetree.unsafe_get_ref(), &cw::tree::line_down));
 
-	    m->add_visible_widget(e, false);
-	    m->add_visible_widget(wt, true);
-	    m->add_visible_widget(lt, true);
-	    m->add_visible_widget(why_table, true);
+	    m->add_widget(e, W_("Hierarchy Editor"));
+
+	    m->add_widget(wt, W_("Description"));
+	    wt->show_all();
+
+	    m->add_widget(lt, W_("Related Dependencies"));
+	    lt->show_all();
+
+	    m->add_widget(why_table, W_("Why Installed"));
+	    why_table->show_all();
+
+	    aptcfg->connect(PACKAGE "::UI::InfoAreaTabs",
+			    sigc::bind(sigc::ptr_fun(&do_update_info_area_show_tabs),
+				       m.weak_ref()));
+	    m->set_show_tabs(aptcfg->FindB(PACKAGE "::UI::InfoAreaTabs", false));
 
 	    if(show_reason_first)
 	      lt->show();
