@@ -23,16 +23,16 @@
 
 #include <aptitude.h>
 
-#include <vscreen/config/colors.h>
-#include <vscreen/config/keybindings.h>
-#include <vscreen/fragment.h>
-#include <vscreen/vs_button.h>
-#include <vscreen/vs_center.h>
-#include <vscreen/vs_frame.h>
-#include <vscreen/vs_scrollbar.h>
-#include <vscreen/vs_table.h>
-#include <vscreen/vs_text_layout.h>
-#include <vscreen/vscreen.h>
+#include <cwidget/config/colors.h>
+#include <cwidget/config/keybindings.h>
+#include <cwidget/fragment.h>
+#include <cwidget/widgets/button.h>
+#include <cwidget/widgets/center.h>
+#include <cwidget/widgets/frame.h>
+#include <cwidget/widgets/scrollbar.h>
+#include <cwidget/widgets/table.h>
+#include <cwidget/widgets/text_layout.h>
+#include <cwidget/toplevel.h>
 
 #include <generic/apt/apt.h>
 #include <generic/apt/aptitude_resolver_universe.h>
@@ -43,9 +43,15 @@
 
 #include <sigc++/bind.h>
 
+namespace cw = cwidget;
+namespace cwidget
+{
+  using namespace widgets;
+}
+
 typedef generic_solution<aptitude_universe> aptitude_solution;
 
-class solution_dialog:public vs_text_layout
+class solution_dialog:public cw::text_layout
 {
   aptitude_solution last_sol;
 
@@ -59,7 +65,7 @@ class solution_dialog:public vs_text_layout
 
   void post_update()
   {
-    vscreen_post_event(new slot_event(sigc::mem_fun(this, &solution_dialog::update)));
+    cw::toplevel::post_event(new cw::toplevel::slot_event(sigc::mem_fun(this, &solution_dialog::update)));
   }
 
 protected:
@@ -75,20 +81,20 @@ protected:
   }
 
 public:
-  static ref_ptr<solution_dialog> create()
+  static cw::util::ref_ptr<solution_dialog> create()
   {
-    ref_ptr<solution_dialog> rval(new solution_dialog);
+    cw::util::ref_ptr<solution_dialog> rval(new solution_dialog);
     rval->decref();
     return rval;
   }
 
   void update()
   {
-    vs_widget_ref tmpref(this);
+    cw::widget_ref tmpref(this);
 
     if(!apt_cache_file)
       {
-	set_fragment(fragf("%s", _("The package cache is not available.")));
+	set_fragment(cw::fragf("%s", _("The package cache is not available.")));
 	last_sol.nullify();
 	return;
       }
@@ -96,7 +102,7 @@ public:
     if(!resman->resolver_exists())
       {
 	// This makes ASS-U-MPTIONS about how resolver_exists works.
-	set_fragment(fragf("%s", _("No packages are broken.")));
+	set_fragment(cw::fragf("%s", _("No packages are broken.")));
 	last_sol.nullify();
 	return;
       }
@@ -105,7 +111,7 @@ public:
 
     if(state.solutions_exhausted && state.generated_solutions == 0)
       {
-	set_fragment(fragf("%s", _("No resolution found.")));
+	set_fragment(cw::fragf("%s", _("No resolution found.")));
 	return;
       }
 
@@ -113,12 +119,12 @@ public:
       {
 	if(state.background_thread_aborted)
 	  {
-	    set_fragment(fragf("%s", state.background_thread_abort_msg.c_str()));
+	    set_fragment(cw::fragf("%s", state.background_thread_abort_msg.c_str()));
 	    return;
 	  }
 	else
 	  {
-	    set_fragment(fragf("%s", _("Resolving dependencies...")));
+	    set_fragment(cw::fragf("%s", _("Resolving dependencies...")));
 	    return;
 	  }
       }
@@ -131,86 +137,86 @@ public:
     last_sol=sol;
 
     if(sol.get_actions().empty())
-      set_fragment(fragf("%s", _("Internal error: unexpected null solution.")));
+      set_fragment(cw::fragf("%s", _("Internal error: unexpected null solution.")));
     else
       set_fragment(solution_fragment(sol));
   }
 };
 
-typedef ref_ptr<solution_dialog> solution_dialog_ref;
+typedef cw::util::ref_ptr<solution_dialog> solution_dialog_ref;
 
-static void do_apply(vscreen_widget &wBare)
+static void do_apply(cw::widget &wBare)
 {
-  vs_widget_ref w(&wBare);
+  cw::widget_ref w(&wBare);
 
   do_apply_solution();
   w->destroy();
 }
 
-vs_widget_ref make_solution_dialog()
+cw::widget_ref make_solution_dialog()
 {
-  vs_table_ref t=vs_table::create();
-  vs_widget_ref rval=vs_center::create(vs_frame::create(t));
+  cw::table_ref t=cw::table::create();
+  cw::widget_ref rval=cw::center::create(cw::frame::create(t));
 
-  vs_text_layout_ref display=solution_dialog::create();
-  vs_scrollbar_ref scrl=vs_scrollbar::create(vs_scrollbar::VERTICAL);
+  cw::text_layout_ref display=solution_dialog::create();
+  cw::scrollbar_ref scrl=cw::scrollbar::create(cw::scrollbar::VERTICAL);
 
-  display->location_changed.connect(sigc::mem_fun(scrl.unsafe_get_ref(), &vs_scrollbar::set_slider));
-  scrl->scrollbar_interaction.connect(sigc::mem_fun(display.unsafe_get_ref(), &vs_text_layout::scroll));
+  display->location_changed.connect(sigc::mem_fun(scrl.unsafe_get_ref(), &cw::scrollbar::set_slider));
+  scrl->scrollbar_interaction.connect(sigc::mem_fun(display.unsafe_get_ref(), &cw::text_layout::scroll));
 
   t->add_widget_opts(display,
 		     0, 0, 1, 1,
-		     vs_table::EXPAND | vs_table::FILL | vs_table::SHRINK,
-		     vs_table::EXPAND | vs_table::FILL | vs_table::SHRINK);
+		     cw::table::EXPAND | cw::table::FILL | cw::table::SHRINK,
+		     cw::table::EXPAND | cw::table::FILL | cw::table::SHRINK);
 
   t->add_widget_opts(scrl,
 		     0, 1, 1, 1,
-		     vs_table::ALIGN_RIGHT,
-		     vs_table::ALIGN_CENTER | vs_table::FILL | vs_table::SHRINK);
+		     cw::table::ALIGN_RIGHT,
+		     cw::table::ALIGN_CENTER | cw::table::FILL | cw::table::SHRINK);
 
-  vs_table_ref bt=vs_table::create();
+  cw::table_ref bt=cw::table::create();
 
   //t->set_bg(get_color("DefaultWidgetBackground")|A_REVERSE);
 
   // TODO: for dialogs like this, I really should have support for
   // "wrapping" lines of buttons if they get too long, like fragments.
-  vs_button_ref bprev  = vs_button::create(_("Previous"));
-  vs_button_ref bnext  = vs_button::create(_("Next"));
-  vs_button_ref bapply = vs_button::create(_("Apply"));
-  vs_button_ref bclose = vs_button::create(_("Close"));
+  cw::button_ref bprev  = cw::button::create(_("Previous"));
+  cw::button_ref bnext  = cw::button::create(_("Next"));
+  cw::button_ref bapply = cw::button::create(_("Apply"));
+  cw::button_ref bclose = cw::button::create(_("Close"));
 
   bprev->pressed.connect(sigc::ptr_fun(do_previous_solution));
   bnext->pressed.connect(sigc::ptr_fun(do_next_solution));
   bapply->pressed.connect(sigc::bind(sigc::ptr_fun(do_apply),
 				     rval.weak_ref()));
   bclose->pressed.connect(sigc::mem_fun(rval.unsafe_get_ref(),
-					&vscreen_widget::destroy));
+					&cw::widget::destroy));
 
-  rval->connect_key("ApplySolution", &global_bindings,
+  rval->connect_key("ApplySolution", &cw::config::global_bindings,
 		    sigc::bind(sigc::ptr_fun(do_apply),
 			       rval.weak_ref()));
 
-  bprev->set_bg_style(style_attrs_flip(A_REVERSE));
-  bnext->set_bg_style(style_attrs_flip(A_REVERSE));
-  bapply->set_bg_style(style_attrs_flip(A_REVERSE));
-  bclose->set_bg_style(style_attrs_flip(A_REVERSE));
+  bprev->set_bg_style(cw::style_attrs_flip(A_REVERSE));
+  bnext->set_bg_style(cw::style_attrs_flip(A_REVERSE));
+  bapply->set_bg_style(cw::style_attrs_flip(A_REVERSE));
+  bclose->set_bg_style(cw::style_attrs_flip(A_REVERSE));
 
   bt->add_widget_opts(bprev,
 		     0, 0, 1, 1,
-		     vs_table::FILL | vs_table::EXPAND, vs_table::FILL);
+		     cw::table::FILL | cw::table::EXPAND, cw::table::FILL);
   bt->add_widget_opts(bnext,
 		     0, 1, 1, 1,
-		     vs_table::FILL | vs_table::EXPAND, vs_table::FILL);
+		     cw::table::FILL | cw::table::EXPAND, cw::table::FILL);
   bt->add_widget_opts(bapply,
 		     0, 2, 1, 1,
-		     vs_table::FILL | vs_table::EXPAND, vs_table::FILL);
+		     cw::table::FILL | cw::table::EXPAND, cw::table::FILL);
   bt->add_widget_opts(bclose,
 		     0, 3, 1, 1,
-		     vs_table::FILL | vs_table::EXPAND, vs_table::FILL);
+		     cw::table::FILL | cw::table::EXPAND, cw::table::FILL);
 
   t->add_widget_opts(bt, 1, 0, 1, 2,
-		     vs_table::FILL | vs_table::EXPAND | vs_table::SHRINK,
-		     vs_table::FILL);
+		     cw::table::FILL | cw::table::EXPAND | cw::table::SHRINK,
+		     cw::table::FILL);
 
   return rval;
 }
