@@ -1,6 +1,6 @@
 // solution_item.cc
 //
-//   Copyright (C) 2005 Daniel Burrows
+//   Copyright (C) 2005, 2007 Daniel Burrows
 //
 //   This program is free software; you can redistribute it and/or
 //   modify it under the terms of the GNU General Public License as
@@ -32,13 +32,18 @@
 
 #include <generic/util/util.h>
 
-#include <vscreen/config/keybindings.h>
-#include <vscreen/fragment.h>
-#include <vscreen/transcode.h>
-#include <vscreen/vs_tree.h>
+#include <cwidget/config/keybindings.h>
+#include <cwidget/fragment.h>
+#include <cwidget/generic/util/transcode.h>
+#include <cwidget/widgets/tree.h>
 
 
 using namespace std;
+namespace cw = cwidget;
+namespace cwidget
+{
+  using namespace widgets;
+}
 
 
 action_type analyze_action(const aptitude_universe::version &ver)
@@ -81,44 +86,44 @@ action_type analyze_action(const aptitude_universe::version &ver)
 /** \return a description of what will happen if the given version
  *  is installed.
  */
-static fragment *action_description(const aptitude_resolver_version &ver)
+static cw::fragment *action_description(const aptitude_resolver_version &ver)
 {
   pkgCache::PkgIterator pkg = ver.get_pkg();
 
   switch(analyze_action(ver))
     {
     case action_remove:
-      return fragf(_("Remove %F [%s (%s)]"),
-		   text_fragment(pkg.Name(), style_attrs_on(A_BOLD)),
+      return cw::fragf(_("Remove %F [%s (%s)]"),
+		   cw::text_fragment(pkg.Name(), cw::style_attrs_on(A_BOLD)),
 		   pkg.CurrentVer().VerStr(),
 		   archives_text(pkg.CurrentVer()).c_str());
       break;
 
     case action_install:
-      return fragf(_("Install %F [%s (%s)]"),
-		   text_fragment(pkg.Name(), style_attrs_on(A_BOLD)),
+      return cw::fragf(_("Install %F [%s (%s)]"),
+		   cw::text_fragment(pkg.Name(), cw::style_attrs_on(A_BOLD)),
 		   ver.get_ver().VerStr(),
 		   archives_text(ver.get_ver()).c_str());
       break;
 
     case action_keep:
       if(ver.get_ver().end())
-	return fragf(_("Cancel the installation of %F"),
-		     text_fragment(pkg.Name(), style_attrs_on(A_BOLD)));
+	return cw::fragf(_("Cancel the installation of %F"),
+		     cw::text_fragment(pkg.Name(), cw::style_attrs_on(A_BOLD)));
       else if(ver.get_package().current_version().get_ver().end())
-	return fragf(_("Cancel the removal of %F"),
-		     text_fragment(pkg.Name(), style_attrs_on(A_BOLD)));
+	return cw::fragf(_("Cancel the removal of %F"),
+		     cw::text_fragment(pkg.Name(), cw::style_attrs_on(A_BOLD)));
       else
-	return fragf(_("Keep %F at version %s (%s)"),
-		     text_fragment(pkg.Name(), style_attrs_on(A_BOLD)),
+	return cw::fragf(_("Keep %F at version %s (%s)"),
+		     cw::text_fragment(pkg.Name(), cw::style_attrs_on(A_BOLD)),
 		     ver.get_ver().VerStr(),
 		     archives_text(ver.get_ver()).c_str());
 
       break;
 
     case action_upgrade:
-      return fragf(_("Upgrade %F [%s (%s) -> %s (%s)]"),
-		   text_fragment(pkg.Name(), style_attrs_on(A_BOLD)),
+      return cw::fragf(_("Upgrade %F [%s (%s) -> %s (%s)]"),
+		   cw::text_fragment(pkg.Name(), cw::style_attrs_on(A_BOLD)),
 		   pkg.CurrentVer().VerStr(),
 		   archives_text(pkg.CurrentVer()).c_str(),
 		   ver.get_ver().VerStr(), archives_text(ver.get_ver()).c_str());
@@ -126,8 +131,8 @@ static fragment *action_description(const aptitude_resolver_version &ver)
 
 
     case action_downgrade:
-      return fragf(_("Downgrade %F [%s (%s) -> %s (%s)]"),
-		   text_fragment(pkg.Name(), style_attrs_on(A_BOLD)),
+      return cw::fragf(_("Downgrade %F [%s (%s) -> %s (%s)]"),
+		   cw::text_fragment(pkg.Name(), cw::style_attrs_on(A_BOLD)),
 		   pkg.CurrentVer().VerStr(), archives_text(pkg.CurrentVer()).c_str(),
 		   ver.get_ver().VerStr(), archives_text(ver.get_ver()).c_str());
       break;
@@ -150,30 +155,30 @@ const wchar_t *solution_item::label()
   return L"";
 }
 
-style solution_item::get_normal_style()
+cw::style solution_item::get_normal_style()
 {
   if(is_rejected())
-    return get_style("SolutionActionRejected");
+    return cw::get_style("SolutionActionRejected");
   else if(is_mandatory())
-    return get_style("SolutionActionApproved");
+    return cw::get_style("SolutionActionApproved");
   else
-    return style();
+    return cw::style();
 }
 
-bool solution_item::dispatch_key(const key &k, vs_tree *owner)
+bool solution_item::dispatch_key(const cw::config::key &k, cw::tree *owner)
 {
-  if(global_bindings.key_matches(k, "SolutionActionReject"))
+  if(cw::config::global_bindings.key_matches(k, "SolutionActionReject"))
     {
       toggle_rejected();
       owner->line_down();
     }
-  else if(global_bindings.key_matches(k, "SolutionActionApprove"))
+  else if(cw::config::global_bindings.key_matches(k, "SolutionActionApprove"))
     {
       toggle_mandated();
       owner->line_down();
     }
   else
-    return vs_treeitem::dispatch_key(k, owner);
+    return cw::treeitem::dispatch_key(k, owner);
 
   return true;
 }
@@ -240,37 +245,39 @@ void solution_act_item::unmandate()
   resman->unmandate_version(ver);
 }
 
-void solution_act_item::highlighted(vs_tree *win)
+void solution_act_item::do_highlighted_changed(bool highlighted)
 {
-  if(apt_cache_file == NULL)
+  if(highlighted)
     {
-      set_short_description(fragf(""));
-      set_active_dep(aptitude_resolver_dep());
-      return;
+      if(apt_cache_file == NULL)
+	{
+	  set_short_description(cw::fragf(""));
+	  set_active_dep(aptitude_resolver_dep());
+	  return;
+	}
+
+      pkgCache::VerIterator real_ver = ver.get_ver();
+
+      if(real_ver.end())
+	real_ver = ver.get_package().current_version().get_ver();
+
+      if(real_ver.end())
+	real_ver = ver.get_pkg().VersionList();
+
+      if(real_ver.end() || real_ver.FileList().end() ||
+	 apt_package_records == NULL)
+	set_short_description(cw::fragf(""));
+      else
+	set_short_description(cw::text_fragment(get_short_description(real_ver, apt_package_records)));
+
+      set_active_dep(d);
     }
-
-  pkgCache::VerIterator real_ver = ver.get_ver();
-
-  if(real_ver.end())
-    real_ver = ver.get_package().current_version().get_ver();
-
-  if(real_ver.end())
-    real_ver = ver.get_pkg().VersionList();
-
-  if(real_ver.end() || real_ver.FileList().end() ||
-     apt_package_records == NULL)
-    set_short_description(fragf(""));
   else
-    set_short_description(text_fragment(get_short_description(real_ver)));
+    {
+      set_short_description(cw::fragf(""));
 
-  set_active_dep(d);
-}
-
-void solution_act_item::unhighlighted(vs_tree *win)
-{
-  set_short_description(fragf(""));
-
-  set_active_dep(aptitude_resolver_dep());
+      set_active_dep(aptitude_resolver_dep());
+    }
 }
 
 void solution_act_item::show_target_info()
@@ -291,9 +298,9 @@ void solution_act_item::show_target_info()
 		     ssprintf(_("Information about %s"), pkg.Name()));
 }
 
-bool solution_act_item::dispatch_key(const key &k, vs_tree *owner)
+bool solution_act_item::dispatch_key(const cw::config::key &k, cw::tree *owner)
 {
-  if(global_bindings.key_matches(k, "InfoScreen"))
+  if(cw::config::global_bindings.key_matches(k, "InfoScreen"))
     {
       show_target_info();
       return true;
@@ -302,7 +309,7 @@ bool solution_act_item::dispatch_key(const key &k, vs_tree *owner)
     return solution_item::dispatch_key(k, owner);
 }
 
-void solution_act_item::paint(vs_tree *win, int y, bool hierarchical, const style &st)
+void solution_act_item::paint(cw::tree *win, int y, bool hierarchical, const cw::style &st)
 {
   unsigned int basex = hierarchical ? 2*get_depth() : 0;
   unsigned int width = win->getmaxx();
@@ -352,16 +359,16 @@ void solution_act_item::paint(vs_tree *win, int y, bool hierarchical, const styl
       ++x;
     }
 
-  fragment *f = clipbox(action_description(ver));
-  fragment_contents c = f->layout(width-x, width-x, st);
+  cw::fragment *f = clipbox(action_description(ver));
+  cw::fragment_contents c = f->layout(width-x, width-x, st);
   delete f;
 
   eassert(c.size() < 2);
   if(c.size() > 0)
     {
-      const fragment_line &l = c.front();
+      const cw::fragment_line &l = c.front();
 
-      fragment_line::const_iterator loc = l.begin();
+      cw::fragment_line::const_iterator loc = l.begin();
       while(loc != l.end() && x < width)
 	{
 	  win->attrset(loc->attrs);
@@ -394,7 +401,7 @@ bool solution_act_item::view_target()
 
 
 
-void solution_act_item_bare::paint(vs_tree *win, int y, bool hierarchical, const style &st)
+void solution_act_item_bare::paint(cw::tree *win, int y, bool hierarchical, const cw::style &st)
 {
   unsigned int basex = hierarchical ? 2*get_depth() : 0;
   unsigned int width = win->getmaxx();
@@ -426,7 +433,7 @@ void solution_act_item_bare::paint(vs_tree *win, int y, bool hierarchical, const
       ++x;
     }
 
-  win->apply_style(st+style_attrs_on(A_BOLD));
+  win->apply_style(st+cw::style_attrs_on(A_BOLD));
 
   aptitude_universe::version ver = get_ver();
 
@@ -522,17 +529,19 @@ bool solution_unresolved_item::is_mandatory()
   return resman->is_approved_broken(d);
 }
 
-void solution_unresolved_item::highlighted(vs_tree *win)
+void solution_unresolved_item::do_highlighted_changed(bool highlighted)
 {
-  if(apt_cache_file == NULL)
-    set_active_dep(aptitude_resolver_dep());
+  if(highlighted)
+    {
+      if(apt_cache_file == NULL)
+	set_active_dep(aptitude_resolver_dep());
+      else
+	set_active_dep(d);
+    }
   else
-    set_active_dep(d);
-}
-
-void solution_unresolved_item::unhighlighted(vs_tree *win)
-{
-  set_active_dep(aptitude_resolver_dep());
+    {
+      set_active_dep(aptitude_resolver_dep());
+    }
 }
 
 void solution_unresolved_item::reject()
@@ -555,7 +564,7 @@ void solution_unresolved_item::unmandate()
   resman->unapprove_broken_dep(d);
 }
 
-void solution_unresolved_item::paint(vs_tree *win, int y, bool hierarchical, const style &st)
+void solution_unresolved_item::paint(cw::tree *win, int y, bool hierarchical, const cw::style &st)
 {
   unsigned int basex = hierarchical ? 2*get_depth() : 0;
   unsigned int width = win->getmaxx();
@@ -590,11 +599,11 @@ void solution_unresolved_item::paint(vs_tree *win, int y, bool hierarchical, con
   wstring text;
 
   if(!fully_explained)
-    text = swsprintf(transcode(_("%s recommends %s")).c_str(),
+    text = swsprintf(W_("%s recommends %s").c_str(),
 		     d.get_dep().ParentPkg().Name(),
 		     dep_targets(d.get_dep()).c_str());
   else
-    text = swsprintf(transcode(_("-> Leave the dependency \"%s recommends %s\" unresolved.")).c_str(),
+    text = swsprintf(W_("-> Leave the dependency \"%s recommends %s\" unresolved.").c_str(),
 		     d.get_dep().ParentPkg().Name(),
 		     dep_targets(d.get_dep()).c_str());
 
